# Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
# Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
"""
Manage global variables and constants
Variables defined here are dedicated to python code
all common definitions should go to globals.sh
"""

import logging
import os.path

# Return code used within the backend for success/failure
SUCCESS = 0
FAILURE = 1
TEMP_EXPORT_BACK = "/tmp/bak_lsa_export"
SAMBA_DIR_ACCESS_MASK = '0770'
SERVICE_SAMBA_SMB = 'smb'
SERVICE_SAMBA_NMB = 'nmb'
USER_NOBODY = 'nobody'
SAMBA_SERVICES = [SERVICE_SAMBA_SMB, SERVICE_SAMBA_NMB]
LOG_DIR = "/opt/lsa/webapp/logs/"
LOG_FILE_NAME = LOG_DIR + "/lsa.log"
VAR_DIR = "/opt/lsa/var"
PRIV_EXEC_LOG_FILENAME = LOG_DIR + "/lsa-priv-exec.log"
TASK_MGR_DB_FILE = "taskmgr.db"
LSA_DEFAULT_FS = "xfs"
BACKUP_FSTAB = "/etc/fstab.bak.lsa"
TEMP_FSTAB_BACK = "/tmp/fstab"
UPTRACK_CONF = "/etc/uptrack/uptrack.conf"
LV_DEFAULT_SECTOR_SIZE = 512
LV_DEFAULT_PHYSICAL_EXTENT_SIZE = 4194304
OPC_AUTHORIZED_KEY_FILE = "/home/opc/.ssh/authorized_keys"
BAK_OPC_KEY_FILE = OPC_AUTHORIZED_KEY_FILE + ".bak"
TEMP_SSH_KEY_FILE = "/tmp/tmp_ssh_key"
CMD_PASSWD = "/usr/bin/passwd"
CMD_HOSTNAME = "/usr/bin/hostname"
CMD_UNAME = "/usr/bin/uname"
CMD_RPM = "/usr/bin/rpm"
CMD_NETSTAT = "/usr/bin/netstat"
CMD_MKFS = "/usr/sbin/mkfs.xfs"
CMD_BLKID = "/usr/sbin/blkid"
CMD_DD = "/usr/bin/dd"
CMD_VGDISPLAY = "/usr/sbin/vgdisplay"
CMD_XFS_GROWFS = "/usr/sbin/xfs_growfs"
CMD_XFS_FREEZE = "/usr/sbin/xfs_freeze"
CMD_XFS_DUMP = "/usr/sbin/xfsdump"
CMD_XFS_RESTORE = "/usr/sbin/xfsrestore"
CMD_USER_ADD = "/usr/sbin/useradd"
CMD_USER_DEL = "/usr/sbin/userdel"
CMD_IP = "/usr/sbin/ip"
CMD_OPENSSL = "/bin/openssl"
CMD_CHATTR = "/usr/bin/chattr"
CMD_SED = "/usr/bin/sed"
CMD_REALM = "/usr/sbin/realm"

CMD_SMBCONTROL = "/usr/bin/smbcontrol"

LSA_INIT_DIR = "/opt/lsa/backend/init"
LSA_CMD_CREATE_POOL = LSA_INIT_DIR + "/create_pool.sh"
RAID_LEVEL_NONE = 99

ORACLE_RELEASE_FILE = "/etc/oracle-release"
DEV_PATH_BY_ID = "/dev/disk/by-id"
SAFE_SOCKET_OP_TIMEOUT = 600
DOWNLOAD_BUFFER_SIZE = 8192
LSA_DEFAULT_SHARE_SIZE = 107374182400

# session timeout value in second
WEBAPP_ABSOLUTE_TIMEOUT = 86400
# activity timeout value in second
WEBAPP_ACTIVITY_TIMEOUT = 3600

# default lines of logs returned
DEFAULT_LOG_LINES = 2500

OS_STREAM_READ_SIZE = 10485760

STREAM_QUEUE_LENGTH = 3

# default log level
LOG_LEVEL = logging.DEBUG if os.path.isfile('/tmp/.lsa_debug') or os.path.isfile('/var/opt/lsa/.lsa_debug') else logging.WARNING

# now include all from script globals file
execfile(os.path.join(os.path.dirname(__file__), 'init/globals.sh'), globals(), locals())
