# Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
# Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
"""
This module hosts disk related helpers
"""


from backend.common import (SFDISK)
from backend.log_util import get_logger, set_log_config
import subprocess
from backend.command_util import get_output_from_command
LOG = get_logger('lsa.disk_utils')


def does_disk_contain_partition_table(dev_name):
    '''
    Checks that a disk contains partition table
    Parameter :
        dev_name: device name as string
    Returns True if has partition table (disk labelized), False otherwise
    Raises StandardError, in case underlying commands failed
    '''

    # check for partition table type using the sfdisk(8) command
    # If the dump is empty , that means that partition table is missing
    try:
        sfdisk_out = get_output_from_command([SFDISK, '--dump', dev_name])
        if len(sfdisk_out) == 0:
            return False
        return True
    except subprocess.CalledProcessError, e:
        LOG.error("Failed to get disk partition information: %s" % str(e))
        raise StandardError("Failed to get disk partition information: %s" % str(e))


if __name__ == '__main__':
    set_log_config()
