#!/usr/bin/bash
# Copyright (c) 2017, 2018, Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.


#
#    This script holds all general functions that are used by other scripts.
#    The global variables, command paths must be defined in globals.sh
#



: '
    check exit code for a command by checking return code $?
    parameter:
    $1 : command/task description in short
    $2 : want to exit the script upon error. 1: want to exit, other: do not exit
'
function check_exit() {

    local exit_code=$?

    # 2nd parameter is to decide if we want to exit for an error.
    # If the value is 1 then we exits upon error
    ret=$?
    if [ -z "$2" ]; then
	do_exit=0
    else
	do_exit=$2
    fi

    if [ $exit_code -eq 0 ]; then
	echo "$1 successful"
	return 0
    else
	# If error, also redirect to stdout
	echo "$1 failed."
	if [ "$do_exit" == "1" ]; then
	    exit $exit_code
	fi
	return 1
    fi
};

: '
    set system locale to C, to avoid non-ASCII characters from stdout/stderr
'
function set_locale_c() {
    $LOCALECTL set-locale LANG=C
}
script_name=$($BASENAME ${0#-}) #- needed if sourced no path
this_script=$($BASENAME ${BASH_SOURCE})
if [[ ${script_name} = ${this_script} ]]; then
    echo "running me directly, nothing executed"
fi
