#!/bin/bash
# Copyright (c) 2017, 2018, Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.

: '
    This is the main script run by config-lsa.service.
    This includes the following.
    1. Set up firewall rules
    2. Set up systemd services
    3. Set up and configure/adjust NFS/SMB settings
    4. Set up apache web server
    5. Configure selinux for the appliance

'

CUR_DIR="$(dirname "$0")"

source $CUR_DIR/globals.sh
source $CUR_DIR/common.sh
source $CUR_DIR/firewall.sh
source $CUR_DIR/nfs.sh
source $CUR_DIR/selinux.sh
source $CUR_DIR/samba.sh

today=$(date "+%Y-%m-%d %H:%M:%S")
echo "---------------------$today-------------"
main_out=0

set_locale_c

# set shares main mount point 
if [ ! -d ${SHARE_POINT} ]
then
    ${MKDIR} ${SHARE_POINT}
fi

# First  configure things not relying on storage pool
main_setup_firewalld
ret=$?
if [ $ret -ne 0 ]; then
    main_out=$((main_out+$ret))
    echo "Firewall setup failed"
else
    echo "Firewall setup was successful"
fi

main_setup_nfs
ret=$?
if [ $ret -ne 0 ]; then
    main_out=$((main_out+$ret))
    echo "NFS setup failed"
else
    echo "NFS setup was successful"
fi
main_setup_samba
ret=$?
if [ $ret -ne 0 ]; then
    main_out=$((main_out+$ret))
    echo "Samba configuration failed"
else
    echo "Samba configuration was successful"
fi

main_setup_selinux
ret=$?
if [ $ret -ne 0 ]; then
    main_out=$((main_out+$ret))
    echo "selinux rules setup failed"
else
    echo "selinux rules was successful"
fi


if [ $main_out -eq 0 ]; then
    # Successfully configured.
    # Show script execution status and append to log file
    echo "Initial configuration script ran successfully"
    exit 0
else
    # Configuration failed.
    # Show script execution status and append to log file
    echo "Initial configuration script failed"
    exit 1
fi
