#!/usr/bin/bash
# Copyright (c) 2017, 2018, Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.

: '
    This script handles all the firewall related functions.
    1. Adding firewall rules
    2. Adding custom firewall rules
    3. Managing firewall service
    4. And so
'

CUR_DIR="$(dirname "$0")"
source $CUR_DIR/globals.sh
source $CUR_DIR/common.sh

#
#  switch between offline and  online firewalld command
#  set the global flag _firewalld_running
#
_firewalld_cmd=${FRLDCMD}
_firewalld_running=1
: '
  switch firewalld command according to current state
'
function set_firewalld_cmd() {
    ${FRLDCMD} --state >/dev/null 2>&1
    if [ $? -ne 0 ]
    then
	_firewalld_cmd=${FRLDOFFLINECMD}
	_firewalld_running=0
    fi
}

: '
    Reload firewalld service
'
function reload_firewalld() {
    echo "reload firewalld configuration"
    $FRLDCMD --reload
    check_exit "firewall reload"
    return $?
};

: '
    Add firewall rule for a particular service
'
function add_firewalld_service() {
    svc=$1
    if [ -z $svc ]; then
	>&2 "Service name is empty"
	return 1
    fi

    out=0
    _cmd=${_firewalld_cmd}
    ${_cmd} --zone=public --add-service=${svc}
    out=$((out+$?))

    if [ $_firewalld_running -eq 1 ]
    then
    _cmd="${_cmd} --permanent"
    echo "EXEC ${_cmd} --zone=public --add-service=${svc}"
    ${_cmd} --zone=public --add-service=${svc}
    out=$((out+$?))
    fi

    if [ "$out" == "0" ] ; then
    echo "Opening $svc service completed successfully"
    return 0
    fi

    echo "Opening $svc service failed"
    return 1
};

: '
    Add firewall rules for services needed for the appliance
'
function add_appliance_services() {
    out=0
    add_firewalld_service nfs
    check_exit "firewall add-service nfs "
    out=$((out+$?))
    add_firewalld_service mountd
    check_exit "firewall add-service mountd"
    out=$((out+$?))
    add_firewalld_service rpc-bind
    check_exit "firewall add-service rpc-bind"
    out=$((out+$?))
    add_firewalld_service webapp-lsa
    check_exit "firewall add-service webapp-lsa"
    out=$((out+$?))
    add_firewalld_service https
    check_exit "firewall add-service https"
    out=$((out+$?))
    add_firewalld_service samba
    check_exit "firewall add-service samba"
    out=$((out+$?))
    if [ "$out" == "0" ]; then
	return 0
    fi
    return 1
};

: '
    This method might by called by other scripts who has a reference to this script.
'
function main_setup_firewalld() {
    echo "Starting firewall setup"
    set_firewalld_cmd
    out=0
    add_appliance_services
    out=$((out+$?))
    if [ $out -eq 0 ]; then
	echo "Firewall setup was successful"
    else
	echo "Firewall setup failed"
    fi
    if [ "$out" == "0" ]; then
	return 0
    fi
    return 1
};

script_name=$($BASENAME ${0#-}) #- needed if sourced no path
this_script=$($BASENAME ${BASH_SOURCE})

# If running this script should call the main function,
# Other wise script is used as a source
if [[ ${script_name} = ${this_script} ]] ; then
    main_setup_firewalld
    if [ $? -eq 0 ]; then
	echo "Firewall setup was successful"
	exit 0
    fi
    echo "Firewall setup failed"
    exit 1
fi
