#!/usr/bin/bash
# Copyright (c) 2018, Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.

: '
    This script provides functionalities to get oci platform information
    This script can not be called independently and is designed solely to be used as a source
'
CUR_DIR="$(dirname "$0")"
source $CUR_DIR/globals.sh
source $CUR_DIR/common.sh

: '
  Sets the platform state.
  Appliance may have many possible states from 'unconfigured'
  to 'ready'
  param:
	$1 state (one of the LSA_STATE_* values)
  return:
	exit code of underlying script
'
function set_platform_state() {
    /opt/lsa/backend/init/state.sh $1
    return $?
}
: '
  Gets the platform state
  return:
	exit on one of the code defined in globals.sh
'
function get_platform_state() {
    /opt/lsa/backend/init/state.sh
    return $?
}
: '
Check if the shape is supported for raid setup i.e have attached local nvme devices.
Typically we only create raid if
the shape is BM DenseIO or VM DenseIO
Output: zero or one 0/1
1: if the shape is supported for raid creation
2: if the output does not match with the desired format
0: otherwise
'
function is_shape_supported(){
    echo "Getting the oci metadata info to determine if the shape is supported for raid setup"
    inp=$($CURL $OCI_INSTANCE_SHAPE)
    # regex for BM/VM.shape(Standard/DenseIO)X.X
    regex="([A-Z]+).([a-zA-Z]+)([0-9].[0-9])*"
    if [[ $inp =~ $regex ]]; then
	# get the machine type BM or VM
	m_type="${BASH_REMATCH[1]}"
	# get the shape Standard/DenseIO/other
	shape="${BASH_REMATCH[2]}"
	if [[ "$m_type" == "BM" && "$shape" == "DenseIO" ]]; then
	    echo "BM DenseIO, supported for raid setup"
	    return 1
	elif [[ "$m_type" == "VM" && "$shape" != "DenseIO" ]]; then
	    echo "VM but not DenseIO, not supported for raid setup"
	    return 0
	elif [[ "$m_type" == "VM" && "$shape" == "DenseIO" ]]; then
	    echo "VM DenseIO, Supported for raid setup"
	    return 1
	elif [[ "$m_type" == "BM" && "$shape" != "DenseIO" ]]; then
	    echo "BM not DenseIO , not supported for raid setup"
	    return 0
	else
	    echo ""
	    return 2
	fi
    else
	return 2
    fi

};
