#!/bin/bash
# Copyright (c) 2018, Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.

: '
    This script will either configure the storage pool or import it

    We assume that we are executed only if /.lsa is not currently
    mounted on the system. I.e we have to create a pool or we have
    to import one

    Action taken in that order :
      IF <LSA volume group available but not active>
      THEN
	<import the VG and setup the pool>
      ELSE
	IF <platform supported for automatic setup>
	THEN
	   <check for available device and create the pool>
	ELSE
	   <delegate to pool creation UI>
	FI
      FI
'

CUR_DIR="$(dirname "$0")"

source $CUR_DIR/globals.sh
source $CUR_DIR/common.sh
source $CUR_DIR/platform.sh
source $CUR_DIR/storage.sh
source $CUR_DIR/nfs.sh
source $CUR_DIR/samba.sh

main_out=0

does_vg_exist $VG_NAME
if [ $? -eq 0 ]
then
    echo "LSA volume group not detected, creating it"
    #try to create the pool
    is_shape_supported
    storage_supported=$?
    if [[ "$storage_supported" != "1" ]]; then
	echo "Shape does not have attached devices i.e not DenseIO shape, not creating raid array."
	echo "Storage pool creation deferred"
	set_platform_state ${LSA_STATE_PENDING}
	# this is not an error
	exit 0
    else
	devices=($(find_local_nvme_devices))
	if [ $? -ne 0 ]; then
	    echo "Storage setup failed: error looking for local storage"
	else
	    echo "Total disks found in this system (${#devs[@]}): ${devs[@]}"
	    echo "Number of unused NVMe disks ($no_disk): ${final[@]}"
	    select_raid_level ${#devices[@]}
	    raid_level=$?
	    echo "RAID level selected: ${raid_level}"
	    create_storage $raid_level ${devices[@]}
	    if [ $? -ne 0 ]; then
		echo "Storage setup (raid, volume group) failed"
		exit 1
	    else
		echo "Storage setup (raid, volume group) was successful"
		echo "share_point:$SHARE_POINT"
	    fi
	fi
    fi
fi

create_storage_configuration
if [ $? -ne 0 ]
then
    echo "LSA volume group importation has failed"
    exit 1
fi

sync_fstab
if [ $? -ne 0 ]
then
    echo "Cannot sync fstab"
    exit 1
fi

mount_all
if [ $? -ne 0 ]
then
    echo "Cannot mount all shares"
    exit 1
fi

# now configure what not rely on storage pool

setup_nfs_exports_config
ret=$?
if [ $ret -ne 0 ]; then
    main_out=$((main_out+$ret))
    echo "NFS export setup failed"
else
    echo "NFS exports was successful"
fi
sync_nfs_exports
ret=$?
if [ $ret -ne 0 ]; then
    main_out=$((main_out+$ret))
    echo "export of all NFS resources failed"
else
    echo "export of all NFS resources was successful"
fi

setup_samba_pool_config
ret=$?
if [ $ret -ne 0 ]; then
    main_out=$((main_out+$ret))
    echo "samba pool export setup failed"
else
    echo "samba pool export setup was successful"
fi
setup_samba_pool_se_config
ret=$?
if [ $ret -ne 0 ]; then
    main_out=$((main_out+$ret))
    echo "samba pool selinux setup failed "
else
    echo "samba pool selinux setup  was successful"
fi
setup_samba_shares_se_config
ret=$?
if [ $ret -ne 0 ]; then
    main_out=$((main_out+$ret))
    echo "samba exports selinux setup failed "
else
    echo "samba exports selinux setup  was successful"
fi

if [ $main_out -eq 0 ]; then
    # Successfully configured.
    echo "Initial configuration script ran successfully"
    # set platform state
    set_platform_state ${LSA_STATE_READY}
    exit 0
else
    # Configuration failed.
    echo "Initial configuration script failed"
    exit 1
fi
