# Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
# Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
"""
Manage log
"""
import logging
import sys

from logging.handlers import RotatingFileHandler
from logging import Formatter
from backend.common import LOG_LEVEL, LOG_FILE_NAME

LOG_FILE_BK_COUNT = 10
set_log_config_done = set()


def set_log_config(logFile=LOG_FILE_NAME, bkCount=LOG_FILE_BK_COUNT):
    '''
    logFile: LOG_FILE_NAME or PRIV_EXEC_LOG_FILENAME
    bkCount: number of backup log files
    '''
    global set_log_config_done

    if logFile in set_log_config_done:
        return
    set_log_config_done.add(logFile)

    # the code is shared between WSGI backend and priv exec
    # plus it can be run as standalone.
    # Whatever, if 'lsa' logger exists already
    # we skip everything

    try:

        handler = RotatingFileHandler(filename=logFile, mode='a',
                                      maxBytes=10485760, backupCount=bkCount)
        handler.setFormatter(
            Formatter('%(process)d/%(threadName)s %(levelname)s -- %(asctime)s [%(filename)s:%(lineno)s - %(funcName)s()] %(message)s'))

        # keep it simple for now and hardcoded DEBUG level
        handler.setLevel(LOG_LEVEL)
        # we are called because we run as standalone
        # application,we have to setup our root logger
        logging.getLogger('lsa').setLevel(LOG_LEVEL)
        logging.getLogger('lsa').addHandler(handler)

    except IOError:
        print >> sys.stderr, 'cannot init logging module', sys.exc_value


def get_logger(name):

    # all loggers should be child of 'lsa' logger
    # to insure common logging configuration is used.
    # centralized append of 'lsa.' here
    if not name.startswith('lsa.'):
        return logging.getLogger('lsa.' + name)
    return logging.getLogger(name)
