# Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
# Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
"""
This module retrieves system configuration information and
update settings.
Usage:
import backend.system_util
# get auto enable setting of ksplice
state = backend.system_util.get_ksplice_auto_enable_setting()
# toggle current settings
try:
    system_util.update_ksplice_setting(cur_state)
except Exception as e:
    LOG.error('XXXX')
"""

import os
import backend.command_util
from backend.common import UPTRACK_CONF, CMD_SED, AWK


def get_ksplice_auto_enable_setting():
    '''
    Get the setting of ksplice whether is it enabled or not.
    Output: yes or no
    Raise: Standard error if command execution fails
    '''
    if not os.path.exists(UPTRACK_CONF):
        raise IOError('Uptrack configuration file not found')

    cmd = AWK + " '/^autoinstall/{print $NF}' " + UPTRACK_CONF
    out = None
    try:
        out = backend.command_util.output_shell_command(cmd)
    except Exception as e:
        raise StandardError('Failed to get ksplice setting: %s' % str(e))
    return out.replace('\n', '')


def toggle_ksplice_autoupdate_setting(cur_state):
    '''
    toggle the setting of ksplice auto update
    param:
        cur_state: current state of the autoupdate setting
        raise: StandardError if update fails
    '''
    state = get_ksplice_auto_enable_setting()
    next_state = None
    if state != cur_state:
        raise StandardError('Invalid request')
    if cur_state == 'yes':
        cmd1 = CMD_SED + " -e '/^upgrade_on_reboot/ s/^#*/#/' -i " + UPTRACK_CONF
        cmd2 = CMD_SED + " -i 's/^autoinstall =.*/autoinstall = no/' " + UPTRACK_CONF
        next_state = 'no'
    if cur_state == 'no':
        cmd1 = "sed -i '/upgrade_on_reboot/ s/# *//' -i " + UPTRACK_CONF
        cmd2 = "sed -i 's/^autoinstall =.*/autoinstall = yes/' " + UPTRACK_CONF
        next_state = 'yes'
    try:
        # We don't care about the output. There will be update only.
        # Exception will be caught if fails
        backend.command_util.output_shell_command(cmd1)
        backend.command_util.output_shell_command(cmd2)
    except Exception as e:
        raise StandardError('Failed to update ksplice settings: %s' % str(e))
    # update done. Retrieve the updated setting and validate it.
    updated_state = get_ksplice_auto_enable_setting()
    if updated_state != next_state:
        raise StandardError('Failed to update ksplice settings: target state and updated state not same')
    return updated_state
