/**
 * Copyright (c) 2017, 2018 Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */
'use strict';

var _module_base = '../../../jet/static';

var _minPaths = {
    'knockout': _module_base + '/js/libs/knockout/knockout-3.4.2',
    'jquery': _module_base +  '/js/libs/jquery/jquery-3.3.1.min',
    'jqueryui-amd': _module_base + '/js/libs/jquery/jqueryui-amd-1.12.1.min',
    'promise': _module_base + '/js/libs/es6-promise/es6-promise.min',
    'hammerjs': _module_base + '/js/libs/hammer/hammer-2.0.8.min',
    'ojdnd': _module_base + '/js/libs/dnd-polyfill/dnd-polyfill-1.0.0.min',
    'ojs': _module_base + '/js/libs/oj/v5.0.0/min',
    'ojL10n': _module_base + '/js/libs/oj/v5.0.0/ojL10n',
    'ojtranslations': _module_base + '/js/libs/oj/v5.0.0/resources',
    'text': _module_base + '/js/libs/require/text',
    'signals': _module_base + '/js/libs/js-signals/signals.min',
    'customElements': _module_base + '/js/libs/webcomponents/custom-elements.min',
    'proj4': _module_base + '/js/libs/proj4js/dist/proj4-src',
    'css': _module_base + '/js/libs/require-css/css'
};
var _debugPaths = {
    'knockout': _module_base + '/js/libs/knockout/knockout-3.4.2.debug',
    'jquery': _module_base + '/js/libs/jquery/jquery-3.3.1',
    'jqueryui-amd': _module_base + '/js/libs/jquery/jqueryui-amd-1.12.1',
    'promise': _module_base + '/js/libs/es6-promise/es6-promise',
    'hammerjs': _module_base + '/js/libs/hammer/hammer-2.0.8',
    'ojdnd': _module_base + '/js/libs/dnd-polyfill/dnd-polyfill-1.0.0',
    'ojs': _module_base + '/js/libs/oj/v5.0.0/debug',
    'ojL10n': _module_base + '/js/libs/oj/v5.0.0/ojL10n',
    'ojtranslations': _module_base + '/js/libs/oj/v5.0.0/resources',
    'text': _module_base + '/js/libs/require/text',
    'signals': _module_base + '/js/libs/js-signals/signals',
    'customElements': _module_base + '/js/libs/webcomponents/custom-elements.min',
    'proj4': _module_base + '/js/libs/proj4js/dist/proj4-src',
    'css': _module_base + '/js/libs/require-css/css'
};


requirejs.config(
{
    paths:(window.sessionStorage.getItem('lsa.debug') !== null)?_debugPaths:_minPaths,
    waitSeconds: 60,
    // Shim configurations for modules that do not expose AMD
    shim:
    {
        'jquery':
        {
            exports: ['jQuery', '$']
        }
    }
}
);

/**
 * A top-level require call executed by the Application.
 * Although 'ojcore' and 'knockout' would be loaded in any case (they are specified as dependencies
 * by the modules themselves), we are listing them explicitly to get the references to the 'oj' and 'ko'
 * objects in the callback
 */
require(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojknockout',
  'ojs/ojmodule', 'ojs/ojrouter', 'ojs/ojnavigationlist', 'ojs/ojbutton', 'ojs/ojtoolbar'],
        function (oj, ko, $,app) { // this callback gets executed when all required modules are loaded

    $(function() {

      if (window.sessionStorage.getItem('lsa.debug') !== null) {
          oj.Logger.option('level',  oj.Logger.LEVEL_LOG);
          oj.Assert.forceDebug();
      }

      function init() {
        oj.Router.sync().then(
          function () {
              // IE has a feature which saves the webpage data for faster viewing, so that http
              // 'GET' requests don't go to backend if previous 'GET' to same address has been
              // called, this makes some of the views not consitent when the data for the 'GET' has
              // been updated. To overcome this, add an ajax option to disable the cache.
              // see CR27649905
              var msie = (window.navigator.appName == 'Microsoft Internet Explorer');
              var trident = window.navigator.userAgent.indexOf('Trident'); //for IE11

              if (msie || trident >= 0)  // If Internet Explorer
              {
                  $.ajaxSetup({ cache: false });
              }

              // Bind your ViewModel for the content of the whole page body.
              ko.applyBindings(app, document.getElementById('lsapage'));
              $(document).ajaxError(function(event, jqXHR,
                                             ajaxSettings,
                                             thrownError) {
                  if (jqXHR.status === 401) {
                      window.location.replace("/");
                  }
              });

              $('document').ready(function(){
                  // Method for adjusting the content area top/bottom paddings to avoid overlap with any fixed regions.
                  // This method should be called whenever your fixed region height may change.  The application
                  // can also adjust content paddings with css classes if the fixed region height is not changing between
                  // views.
                  var topElem = document.getElementsByClassName('oj-applayout-fixed-top')[0];
                  var contentElem = document.getElementsByClassName('oj-web-applayout-content')[0];
                  var bottomElem = document.getElementsByClassName('oj-applayout-fixed-bottom')[0];

                  if (topElem) {
                      contentElem.style.paddingTop = topElem.offsetHeight+'px';
                  }
                  if (bottomElem) {
                      contentElem.style.paddingBottom = bottomElem.offsetHeight+'px';
                  }
              });
          },
          function (error) {
            oj.Logger.error('Error in root start: ' + error.message);
          }
        );
      }
        init();
    });

  }
);
