/**
 * Copyright (c) 2017, 2018, Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */
define(['ojs/ojcore','jquery'],
       function(oj, $) {
           function Platform() {

           }
           /**
            * synchronous call to logout user
            * @return {string} url we should move to
            */
           Platform.prototype.logoutUser = function () {
               var result = $.ajax(
                   {
                       'url':'/login/logout',
                       'async': false
                   }).responseText;
               return result;
           };
           /**
            * Gets system usages/
            * @returns {Promise} a promise which resolve on the following object
            *    {
            *      bytes_recv
            *      bytes_sent
            *      cpu
            *      disk
            *      disk_avail
            *      mem
            *      mem_avail
            *      swap
            *   }
            */
           Platform.prototype.getAllUsages = function() {
               return(
                       new Promise(function(resolve, reject) {
                           $.ajax( {
                               'url':'/lsa/sys/usages',
                                'async': true,
                               'dataType': "json"
                           }).done(
                               function getsysUsagesAjaxDone (data) {
                                   resolve(data);
                               }
                           ).fail(
                               function getsysUsagesAjaxFail (jqxhr, text_status, error_thown) {
                                   reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                               }
                           );
                       })
                   );
           };
           /**
            * Gets list of system services with their state
            * @return {promise} - a promise which resolve on a list of following object
            *  {
            *    desc : description as string
            *    logs : service logs as list of string
            *    name : service name as string
            *    status : service status as string
            *  }
            */
           Platform.prototype.getServices = function() {
               return(
                       new Promise(function(resolve, reject) {
                           $.ajax( {
                               'url':'/lsa/services',
                               'async': true,
                               'dataType': "json"
                           }).done(
                               function getServicesAjaxDone (data) {
                                   resolve(data.list);
                               }
                           ).fail(
                               function getServicesAjaxFail (jqxhr, text_status, error_thown) {
                                   reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                               }
                           );
                       })
                   );
           };
           /**
            * Gets platform information
            * @returns {promise} - resolve on the following object
            *   {
            *     hostname
            *     ip
            *     kernel_version
            *     mem
            *     os_version
            *     platform_version
            *     root_disk_size
            *     uptime
            *   }
            */
           Platform.prototype.getInfos = function() {
               return(
                       new Promise(function(resolve, reject) {
                           $.ajax( {
                               'url':'/lsa/sys',
                               'async': true,
                               'dataType': "json"
                           }).done(
                               function getInfosAjaxDone (data) {
                                   resolve(data);
                               }
                           ).fail(
                               function getInfosAjaxFail (jqxhr, text_status, error_thown) {
                                   reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                               }
                           );
                       })
                   );
           };
           Platform.prototype.getInstanceDetails = function() {
               return(
                       new Promise(function(resolve, reject) {
                           $.ajax( {
                               'url':'/lsa/platform/instance',
                               'async': true,
                               'dataType': "json"
                           }).done(
                               function getInfosAjaxDone (data) {
                                   resolve(data);
                               }
                           ).fail(
                               function getInfosAjaxFail (jqxhr, text_status, error_thown) {
                                   reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                               }
                           );
                       })
                   );
           };
           /**
            * Gets the root filesystem usage
            * @returns {Promise} - a promise which resolve of root filesystem usage in percent
            */
           Platform.prototype.getRootFSUsage = function() {
               return(
                       new Promise(function(resolve, reject) {
                           $.ajax( {
                               'url':'/lsa/storageUsages',
                               'async': true,
                               'dataType': "json"
                           }).done(
                               function getRootFSUsageAjaxDone (data) {
                                   resolve(data['root']);
                               }
                           ).fail(
                               function getRootFSUsageAjaxFail (jqxhr, text_status, error_thown) {
                                   reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                               }
                           );
                       })
                   );
           };
           /**
            * Test remote platform reachability
            * @returns {Boolean} true on success, false otherwise
            */
           Platform.prototype.ping = function() {
               var reachable = false;
               $.ajax( {
                   'url':'/lsa/sys',
                   'async': false,
                   'complete': function ( jqXHR, textStatus ) {
                       oj.Logger.log('status ' + jqXHR.status);
                       if (jqXHR.status === 200) {
                           reachable = true;
                       }
                   }
               });

               return (reachable);

           };
           /**
            * Unconfigure the platform. Trigger all
            * required action to revert the platform to unconfigure state
            */
           Platform.prototype.unconfigure = function() {
                return (new Promise(function(resolve,reject) {
                           $.ajax( {
                               'url':"/lsa/platform/unconfigure",
                               'method': 'POST',
                               'async': true,
                               'contentType': "application/json"
                           }).done(function unconfigureAjaxDone () {
                               resolve();
                           }).fail(function unconfigureAjaxFail (jqxhr, text_status, error_thown) {
                               reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                           });
                       }));

           };
           /**
            * Gets platform local time
            * @returns {String} "<Locale’s appropriate time representation> <Locale’s appropriate date representation> <Time zone name>"
            * example : "12:04:07 07/20/18 CEST"
            */
           Platform.prototype.getPlatformLocalTime = function() {
               return(
                       new Promise(function(resolve, reject) {
                           $.ajax( {
                               'url':'/lsa/platform/localtime',
                               'async': true,
                               'dataType': "json"
                           }).done(
                               function getPlatformLocalTimeAjaxDone (data) {
                                   resolve(data.ltime);
                               }
                           ).fail(
                               function getPlatformLocalTimeAjaxFail (jqxhr, text_status, error_thown) {
                                   reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                               }
                           );
                       })
                   );
           };
           /**
            * Gets platform status
            */
           Platform.prototype.getPlatformStatus = function() {
               return(
                       new Promise(function(resolve, reject) {
                           $.ajax( {
                               'url':'/lsa/platform/status',
                               'async': true,
                               'dataType': "json"
                           }).done(
                               function getPlatformStatusAjaxDone (data) {
                                   resolve(data);
                               }
                           ).fail(
                               function getPlatformStatusAjaxFail (jqxhr, text_status, error_thown) {
                                   reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                               }
                           );
                       })
                   );
           };

           /**
            * Returns the list of ongoing tasks
            * resolve a list of task as
            * 'ID' : unique ID as int
            * 'name': task name as string
            * 'target': the subject of the task (usually share name)
            * 'status' : {
            *       'code': status code (see backend.TaskStatusCode),
            *       'message': status message
            *      },
            * 'states' : [{
            *       'type': state type (see backend.TaskState),
            *       'message': state message ,
            *       'time': timestamp
            *      }
            */
           Platform.prototype.getTaskList = function() {
               return(
                   new Promise(function(resolve, reject) {
                       $.ajax( {
                           'url':'/lsa/tasks',
                               'async': true,
                               'dataType': "json"
                           }).done(
                               function getTaskListAjaxDone (data) {
                                   resolve(data.tasks.map(JSON.parse));
                               }
                           ).fail(
                               function getTaskListAjaxFail (jqxhr, text_status, error_thown) {
                                   reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                               }
                           );
                       })
                   );
           };
           /**
            * Returns the list of currently running tasks
            * resolve a list of task as
            * 'ID' : unique ID as int
            * 'name': task name as string
            * 'target': the subject of the task (usually share name)
            * 'status' : {
            *       'code': status code (see backend.TaskStatusCode),
            *       'message': status message
            *      },
            * 'states' : [{
            *       'type': state type (see backend.TaskState),
            *       'message': state message ,
            *       'time': timestamp
            *      }
            */
           Platform.prototype.getRunningTaskList = function() {
               return(
                   new Promise(function(resolve, reject) {
                       $.ajax( {
                           'url':'/lsa/tasks',
                           'method':'GET',
                           'async': true,
                           'dataType': "json",
                           'data': {
                               'running': "true"
                           }
                           }).done(
                               function getTaskListAjaxDone (data) {
                                   resolve(data.tasks.map(JSON.parse));
                               }
                           ).fail(
                               function getTaskListAjaxFail (jqxhr, text_status, error_thown) {
                                   reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                               }
                           );
                       })
                   );
           };
           /**
            * Returns the ongoing task of given id
            * resolve a list of task as
            * 'ID' : unique ID as int
            * 'name': task name as string
            * 'target': the subject of the task (usually share name)
            * 'status' : {
            *       'code': status code (see backend.TaskStatusCode),
            *       'message': status message
            *      },
            * 'states' : [{
            *       'type': state type (see backend.TaskState),
            *       'message': state message ,
            *       'time': timestamp
            *      }{
            */
           Platform.prototype.getTask = function(taskID) {
               return(
                       new Promise(function(resolve, reject) {
                           $.ajax( {
                               'url':'/lsa/task/' + taskID,
                               'async': true,
                               'dataType': "json"
                           }).done(
                               function getTaskAjaxDone (data) {
                                   resolve(JSON.parse(data.task));
                               }
                           ).fail(
                               function getTaskAjaxFail (jqxhr, text_status, error_thown) {
                                   reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                               }
                           );
                       })
                   );
           };
           /**
            * Returns platform disks
            * resolve a list of disk as
            *  {
            *     disks:[
            *                 {
            *                 name: name of the disk as string
            *                 type: type of the disk as string, ['iscsi' | 'fc' | 'sas' | 'nvmf' |...]
            *                wwn: world wide name as string
            *                 size: size of disk in bytes as int
            *                 state: state of disk as string, ['running' | 'not running']
            *                status: status of disk as dict
            *                   {
            *                       partitioned: status as boolean of whether the disk contains partitions
            *                       mounted: status as boolean of whether the disk or at least one of the partitions(if any) is mounted
            *                   }
            *                 }
            *               ]
            *       }
            */
           Platform.prototype.getDisks = function () {
               return(
                   new Promise(function(resolve, reject) {
                       $.ajax( {
                           'url':'/lsa/platform/disks',
                           'async': true,
                           'dataType': "json"
                       }).done(
                           function getDisksAjaxDone (data) {
                               resolve(JSON.parse(data.disks));
                           }
                       ).fail(
                           function getDisksAjaxFail (jqxhr, text_status, error_thown) {
                               reject(jqxhr.responseJSON ? jqxhr.responseJSON.msg : jqxhr.responseText);
                           }
                       );
                   })
               );
           };

           return Platform;
       });
