/**
 * Copyright (c) 2018, Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */
/**
 * Module that holds common helpers to manipulate task objects
 *  return from the backend
 */

define(['ojs/ojcore'], function(oj) {
    return {
        /**
         * check that a task has ended succesfuly
         * @param {object} task - task objects
         * @return {bool} true if succesful , false otherwise
         */
        isTaskSuccess: function (task) {
            oj.Assert.assert(typeof task !== 'undefined');
            oj.Assert.assert(typeof task.status !== 'undefined');
            return (task.status.code === 0);
        },
        /**
         * Gets the time a task has ended
         * @param {object} task - task objects
         * @return {Number} - the time in milliseconds since EPOC. or undefined if the
         *  task has not properly ended
         */
        getTaskExecutionEnd: function(task) {
            oj.Assert.assert(typeof task !== 'undefined');
            oj.Assert.assert(typeof task.states !== 'undefined');
            var stateTime;
            task.states.forEach(function(s) {
                if(s.type === 'STOPPED') {
                    // backend task timestamp ar in seconds
                    stateTime = s.time * 1000;
                }
            });
            return stateTime;
        },
        /**
         * Gets the time of the most recent succesfully executed task
         * @param {Array<object>} tasks - Array of task objects
         * @return {Number} - the time in milliseconds since EPOC. or undefined if no task
         * has been found
         */
        getMostRecentTaskExecutionEnd: function(tasks) {
            oj.Assert.assert(Array.isArray(tasks));

            var mostRecentStateTime;
            var _mrst;
            tasks.forEach(function(t) {
                _mrst = undefined;
                t.states.forEach(function(s) {
                    if(s.type === 'STOPPED') {
                        // backend task timestamp ar in seconds
                         _mrst = s.time * 1000;
                    }
                });
                if (_mrst !== undefined) {
                    if ((mostRecentStateTime === undefined) ||
                        (_mrst > mostRecentStateTime)) {
                        mostRecentStateTime = _mrst;
                    }
                }
            });
            return mostRecentStateTime;
        }
    };
});
