/**
 * Copyright (c) 2017, 2018 Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */
define(['ojs/ojcore',
        'knockout',
        'jquery',
        'models/backups',
        'models/logs',
        'models/administration',
        'models/nfs',
        'viewModels/common_vm_def',
        'viewModels/confirmation-dialog',
        'viewModels/oci-config-dialog',
        'models/platform',
        'viewModels/add-ssh-key-dialog',
        'viewModels/samba-global-setting-dialog',
        'ojs/ojtabs',
        'ojs/ojlistview',
        'ojs/ojinputtext',
        'ojs/ojselectcombobox',
        'ojs/ojbutton',
        'ojs/ojswitcher',
        'ojs/ojdefer'],
       function(oj, ko, $,backupSVC,logSVC, adminSVC, nfsSVC, CommonVMDefinitons, confirmation,
                OciConfigDialog, Platform, addSshKeyDialog, sambaGlobalSettingDialog) {

            function AdminViewModel() {
                var self = this;

                this.keyword = ko.observable('');
                this.keyword.subscribe(function() {
                    self.fetchLogs(self.keyword());
                });
                this.current_search_label = ko.computed(function() {
                    if (self.keyword().length === 0) {
                        return 'Not filtered';
                    }
                    return 'Filtered on: ' + self.keyword();
                });

                this.logSwitcherSelectedItem = ko.observable("system");

                this.systemLogs = ko.observable('');
                this.bootLogs = ko.observable('');
                this.serviceLogs = ko.observable('');
                this.updateKspliceLabel = ko.observable(oj.Translations.getTranslatedString('Unknown Ksplice state'));
                this.updateKspliceHelp = ko.observable(oj.Translations.getTranslatedString('Unknown Ksplice state'));
                this.kspliceEnableState = ko.observable('');
                this.updateKspliceDisabled = ko.observable(true);

                function updateKspliceState(state) {
                    self.kspliceEnableState(state);
                    if (state == 'yes'){
                        self.updateKspliceLabel(oj.Translations.getTranslatedString('Disable Ksplice'));
                        self.updateKspliceHelp(oj.Translations.getTranslatedString('Click on this button to disable Ksplice auto update'));
                    }
                    else {
                        self.updateKspliceLabel(oj.Translations.getTranslatedString('Enable Ksplice'));
                        self.updateKspliceHelp(oj.Translations.getTranslatedString('Click on this button to enable Ksplice auto update'));
                    }
                };
                new adminSVC().getSystemConfiguration().then( function(data) {
                    var state = data.ksplice_enable_state;
                    self.updateKspliceDisabled(false);
                    updateKspliceState(state);
                });
                this.rebootClicked = function(data, event){
                    nfsSVC.getClientCount().then(function(count) {
                        var msg = self.rebootDialogText;
                        if (count > 0) {
                            msg = count + ' client' +
                                (count > 1 ? 's are ': ' is ') +
                                ' currently connected. ' + msg;
                        }
                        var p = new Platform();
                        confirmation.showDialog({
                            'title':self.rebootDialogTitle,
                            'message': msg,
                            'onOK' : function () {
                                new adminSVC().reboot().then(function () {
                                    confirmation.showDialog({
                                        'title' :  self.waitingTitle,
                                        'message': self.waitingMessage,
                                        'longMsg': self.waitingSubMessage,
                                        'hideFooter' : true,
                                        'showProgress': true
                                    });
                                    // we start looping around to test reachability
                                    window.setInterval(function () {
                                        if(p.ping() === true) {
                                            oj.Logger.info('server back online');
                                            confirmation.hideDialog();
                                            // redirect to login page
                                            window.location.replace("/login");
                                        }
                                    }, 3000);
                                }).catch (function(raeson) {
                                    oj.Logger.warn('fail to trigger reboot: '+ raeson);
                                });
                            }
                        });
                    }).catch (function (raeson) {
                        oj.Logger.warn('fail to get NFS client count: '+ raeson);
                    });
                };
                this.unconfigureClicked = function (data, event) {
                     confirmation.showDialog({
                       'title':self.unconfigureDialogTitle,
                       'message':self.unconfigureDialogText,
                       'onOK' : function () {
                           new Platform().unconfigure();
                       }
                   });

                };
                this.updateClicked = function(data, event){
                   confirmation.showDialog({
                       'title':self.updateDialogTitle,
                       'message':self.updateDialogText,
                       'onOK' : function () {
                            confirmation.showDialog({
                                'hideFooter':true,
                                'showProgress':true,
                                'title': self.updateDialogTitle,
                                'message':self.updateProgressText,
                                'onOK' : function () {
                                }});
                            new adminSVC().update().then(function(data) {
                                confirmation.showDialog({
                                    'notifyOnly': true,
                                    'outputDialog': true,
                                    'longMsg': true,
                                    'title': self.updateDialogResultTitle,
                                    'message': data.content});
                                }, function(err) {
                                confirmation.showDialog({
                                    'notifyOnly':true,
                                    'title': 'update failed',
                                    'message': err});
                            });
                       }
                   });
                };

                this.restartNFSClicked = function(data, event){
                   confirmation.showDialog({
                       'title':self.restartNFSDialogTitle,
                       'message':self.restartNFSDialogText,
                       'onOK' : function () {
                            new adminSVC().restartNFSServices();
                       }
                   });
                };

               this.restartSMBClicked = function(data, event){
                   confirmation.showDialog({
                       'title':self.restartSMBDialogTitle,
                       'message':self.restartSMBDialogText,
                       'onOK' : function () {
                            new adminSVC().restartSMBServices();
                       }
                   });
                };



                this.updateKspliceSetting = function(data, event){
                   var cur_state = self.kspliceEnableState();
                   confirmation.showDialog({
                       'title': self.updateKspliceDialogTitle,
                       'message': self.updateKspliceDialogText,
                       'onOK' : function () {
                            new adminSVC().updateKspliceSetting(
                                cur_state).then (function (data)
                                          {
                                              oj.Logger.info('Ksplice updatedsuccessfully');
                                              confirmation.showDialog({
                                                'notifyOnly':true,
                                                'title': oj.Translations.getTranslatedString('Successful Modification'),
                                                'message': oj.Translations.getTranslatedString('Ksplice setting was successfully updated')
                                            });
                                            updateKspliceState(data.ksplice_enable_state);
                                          }).catch(function (why) {
                                              oj.Logger.error('Updating ksplice  failed: ' + why);
                                              confirmation.showDialog({
                                                  'notifyOnly':true,
                                                  'title': oj.Translations.getTranslatedString('Updating Ksplice failure'),
                                                  'message': why
                                              });
                                            }
                                        );
                                }
                        });
                };

                this.changePasswordClicked = function(data, event){
                    window.location.href = "/setup/changepassword";;
                };

                this.ociServiceConfigClicked = function (data, event) {
                    OciConfigDialog.open();
                };

                this.addSshKeyClicked = function(){
                    addSshKeyDialog.open();
                };

                this.sambaGlobalSettingClicked = function(){
                    sambaGlobalSettingDialog.open();
                };

                this.fetchLogs = function(value)
                {
                    logSVC.getSystemLogs(value).then(function (logs) {
                        self.systemLogs(logs);
                    });
                    logSVC.getBootLogs(value).then(function (logs) {
                        self.bootLogs(logs);
                    });
                    logSVC.getServiceLogs(value).then(function (logs) {
                        self.serviceLogs(logs);
                    });
                };

                this.clearThings = function(){
                        self.keyword('');
                };

                this.handleActivated = function(info)
                {
                    self.fetchLogs(self.keyword());
                };
            }


            $.extend(AdminViewModel.prototype,CommonVMDefinitons.prototype.definitions);


            AdminViewModel.prototype.keyword = ko.observable('');
            AdminViewModel.prototype.serviceLogs = ko.observable('');
            AdminViewModel.prototype.bootLogs = ko.observable('');
            AdminViewModel.prototype.systemLogs = ko.observable('');

            AdminViewModel.prototype.systemActionTitle =
                    oj.Translations.getTranslatedString('System Actions');
            AdminViewModel.prototype.userActionTitle =
                    oj.Translations.getTranslatedString('User Actions');

            AdminViewModel.prototype.systemLogsTitle =
                    oj.Translations.getTranslatedString('System Logs');


            AdminViewModel.prototype.servicesLogsLabel = oj.Translations.getTranslatedString('Services logs');
            AdminViewModel.prototype.bootLogsLabel = oj.Translations.getTranslatedString('Boot logs');
            AdminViewModel.prototype.systemLogsLabel = oj.Translations.getTranslatedString('System logs');

            AdminViewModel.prototype.rebootLabel = oj.Translations.getTranslatedString('Reboot appliance');
            AdminViewModel.prototype.updateLabel = oj.Translations.getTranslatedString('Update appliance');
            AdminViewModel.prototype.unconfigureLabel = oj.Translations.getTranslatedString('Prepare for migration');
            AdminViewModel.prototype.restartNFSLabel = oj.Translations.getTranslatedString('Restart NFS services');
            AdminViewModel.prototype.restartSMBLabel = oj.Translations.getTranslatedString('Restart SMB services');
            AdminViewModel.prototype.changePasswordLabel = oj.Translations.getTranslatedString('Change password');
            AdminViewModel.prototype.addSshKeyLabel = oj.Translations.getTranslatedString('Update ssh keys');
           AdminViewModel.prototype.sambaGlobalSettingLabel = oj.Translations.getTranslatedString('Samba global settings');
           AdminViewModel.prototype.clearFilterLabel = oj.Translations.getTranslatedString('Clear filter');
            AdminViewModel.prototype.ociServiceConfigLabel = oj.Translations.getTranslatedString('Configure OCI service access');
            AdminViewModel.prototype.ociServiceConfigHelp = oj.Translations.getTranslatedString('Configure access to OCI services');
            AdminViewModel.prototype.rebootHelp = oj.Translations.getTranslatedString('Reboot the system. All current client sessions will be disconnected');
            AdminViewModel.prototype.updateHelp = oj.Translations.getTranslatedString('Update the system to the latest available security updates and appliance package version');
            AdminViewModel.prototype.unconfigureHelp = oj.Translations.getTranslatedString('Prepare the appliance storage for safe migration to a new instance. Refer to the help for details on the migration procedure');
            AdminViewModel.prototype.restartNFSHelp = oj.Translations.getTranslatedString('Restart NFS services');
            AdminViewModel.prototype.restartSMBHelp = oj.Translations.getTranslatedString('Restart Samba and related services');
            AdminViewModel.prototype.changePasswordHelp = oj.Translations.getTranslatedString('Change the password of the admin user');
            AdminViewModel.prototype.addSshKeyHelp = oj.Translations.getTranslatedString('Modify the public ssh keys for the default opc user');
            AdminViewModel.prototype.sambaGlobalSettingHelp = oj.Translations.getTranslatedString('Modify Samba global settings');

            AdminViewModel.prototype.unconfigureDialogTitle = oj.Translations.getTranslatedString('Prepare for shape migration');
            AdminViewModel.prototype.unconfigureDialogText = oj.Translations.getTranslatedString('This action will stop the web interface and prepare the instance for shape migration. This operation cannot be undone. Are you sure you want to continue ?');

            AdminViewModel.prototype.updateDialogTitle = oj.Translations.getTranslatedString('Update appliance');
            AdminViewModel.prototype.updateDialogResultTitle = oj.Translations.getTranslatedString('Update results');
            AdminViewModel.prototype.updateDialogText = oj.Translations.getTranslatedString('Are you sure you want to update to the latest version of the appliance and the latest security updates ?');

            AdminViewModel.prototype.rebootDialogTitle = oj.Translations.getTranslatedString('Reboot appliance');
            AdminViewModel.prototype.rebootDialogText = oj.Translations.getTranslatedString('Are you sure you want to reboot ?');

            AdminViewModel.prototype.updateKspliceDialogTitle = oj.Translations.getTranslatedString('Enable Ksplice');
            AdminViewModel.prototype.updateKspliceDialogText = oj.Translations.getTranslatedString('Are you sure you want to update Ksplice settings?');

            AdminViewModel.prototype.restartNFSDialogTitle = oj.Translations.getTranslatedString('Restart NFS services');
            AdminViewModel.prototype.restartNFSDialogText = oj.Translations.getTranslatedString('Are you sure you want to restart services ?');
            AdminViewModel.prototype.restartSMBDialogTitle = oj.Translations.getTranslatedString('Restart SMB services');
            AdminViewModel.prototype.restartSMBDialogText = oj.Translations.getTranslatedString('Are you sure you want to restart SMB services ?');
            AdminViewModel.prototype.updateProgressTitle = oj.Translations.getTranslatedString('Security Update');
            AdminViewModel.prototype.updateProgressText = oj.Translations.getTranslatedString('Updates in progress, re-authentication may be required upon update');



             AdminViewModel.prototype.waitingMessage =
               oj.Translations.getTranslatedString('Please wait while the appliance is rebooting');

           AdminViewModel.prototype.waitingSubMessage =
               oj.Translations.getTranslatedString('You will be redirected once appliance is reachable again');

           AdminViewModel.prototype.waitingTitle =
               oj.Translations.getTranslatedString('Rebooting...');


            return new AdminViewModel();
        }
      );
