/**
 * Copyright (c) 2017, Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */
/*
 * share backup viewModel
 * bind with backup-share-dialog.html view
 */

define(['ojs/ojcore',
        'knockout',
        'jquery',
        'viewModels/common_vm_def',
        'models/backups',
        'models/platform',
        'ojs/ojinputtext',
        'ojs/ojdialog'],
       function (oj, ko, $, CommonVMDefinitons, backupsSVC, Platform)
    {

        function BackupShareViewModel()
        {
            var self = this;


            this.cancelButtonDisabled =  ko.observable(false);

            this.validationTracker = ko.observable();
            this.shouldDisableBackupButton = ko.pureComputed(function () {
                var trackerObj = ko.utils.unwrapObservable(self.validationTracker);
                return trackerObj ? (trackerObj.invalidShown || trackerObj.invalidHidden) : false;
            });


            this.share = ko.observable({});

            this.namespace = ko.observable();
            this.bucketName = ko.observable('');
            this.objectName = ko.observable('');

            /* dedicated to confirmation dialog */
            this.innerDialogTitle = ko.observable('');
            this.innerDialogMessage = ko.observable('');


            /**
             * method called to open share backup dialog
             * @see backup-share-dialog
             * @param entry: share entry object to be used to prepopulate dialog box
             * @public
             */
            this.open = function (entry)
            {
                self.share(entry);
                self.namespace('');
                self.bucketName('');
                self.objectName('');
                $('#bosnsi').ojInputText("reset");
                $('#bosbni').ojInputText("reset");
                $('#boni').ojInputText("reset");
                $(self.dialogSelector).ojDialog('open');
            };
            /**
             * compute the name of the backup
             * name is backups/<share name>/<share name>."new Date().toISOString()
             */
            this.objectNamePlaceHolder = ko.pureComputed(function () {
                return 'backups/'+self.share().name + '/'+self.share().name+ '_'+new Date().toISOString();
            });
            this.handleInnerClick = function ()
            {
                $(self.innerDialogSelector).ojDialog("close");
                $(self.dialogSelector).ojDialog("close");
            };
            this.handleCancelClick = function ()
            {
                $(self.dialogSelector).ojDialog("close");
            };
            this.handleCloseDrawerClick = function ()
            {
                $(self.dialogSelector).ojDialog("close");
            };

            /**
             * Backup a share depending on the
             * current dialog object actionType
             */
            this.handleBackupClick = function (data, event)
            {
                var _o = self.objectName();
                if (_o === undefined || _o === '') {
                    _o = self.objectNamePlaceHolder();
                }

                backupsSVC.backupShareToObjectStore(
                    self.share().name,
                    self.namespace().trim(),
                    self.bucketName().trim(),
                    _o).then (function (payload)
                              {
                                  $(self.dialogSelector).ojDialog("close");
                              }).catch(function (why) {
                                  oj.Logger.error('Backup has failed: ' + why);
                                  self.innerDialogTitle(self.innerDialogKoTitle);
                                  self.innerDialogMessage('backup has failed: ' + why);
                                  $(self.innerDialogSelector).ojDialog("open");
                              });
            };

        }

        $.extend(BackupShareViewModel.prototype,CommonVMDefinitons.prototype.definitions);

        BackupShareViewModel.prototype.optionRequiredHint = oj.Translations.getTranslatedString('You must enter a value');


        BackupShareViewModel.prototype.dialogSelector='#backupShareDialog';
        BackupShareViewModel.prototype.dialogTitle=oj.Translations.getTranslatedString('Backup Share');

        BackupShareViewModel.prototype.backupNSHelpDef=oj.Translations.getTranslatedString('Namespace associated with the bucket. Can be found in the Bucket Details page on the OCI console. Click for more info.');
        BackupShareViewModel.prototype.backupNSHelpSource='https://docs.us-phoenix-1.oraclecloud.com/Content/Object/Tasks/managingobjects.htm?Highlight=namespace';
        BackupShareViewModel.prototype.OSNSLabel=oj.Translations.getTranslatedString('Object Store namespace');
        BackupShareViewModel.prototype.OSNSPlaceholder=oj.Translations.getTranslatedString('Namespace associated with the bucket');

        BackupShareViewModel.prototype.backupBNHelpDef=oj.Translations.getTranslatedString('Name of the existing bucket where the archive will be stored. Click for more info.');
        BackupShareViewModel.prototype.backupBNPlaceholder=oj.Translations.getTranslatedString('Name of the existing Object storage bucket');
        BackupShareViewModel.prototype.backupBNHelpSource='https://docs.us-phoenix-1.oraclecloud.com/Content/Object/Tasks/managingbuckets.htm';
        BackupShareViewModel.prototype.OSBNLabel=oj.Translations.getTranslatedString('Object Store bucket name');

        BackupShareViewModel.prototype.backupHelp=oj.Translations.getTranslatedString('Archive the content of the current share in an OCI object store object. See online help for details.');

        BackupShareViewModel.prototype.objectNameLabel=oj.Translations.getTranslatedString('New archive object name');
        BackupShareViewModel.prototype.onHelpDef=oj.Translations.getTranslatedString('Name of the object to be stored in the OCI object storage bucket. If left blank the default naming scheme (in italic below) will be used. Click for more info.');
        BackupShareViewModel.prototype.onHelpSource='https://docs.us-phoenix-1.oraclecloud.com/Content/Object/Tasks/managingobjects.htm';


        BackupShareViewModel.prototype.innerDialogSelector='#innerbackupShareDialog';
        BackupShareViewModel.prototype.innerDialogOkTitle = 'Backup in progress';
        BackupShareViewModel.prototype.innerDialogKoTitle = 'Backup failed';
        BackupShareViewModel.prototype.innerDialogMessage1 = 'Share has been dumped successfuly, upload of ';
        BackupShareViewModel.prototype.innerDialogMessage2 = ' is in progress';

        return new BackupShareViewModel();
    }
);
