/**
 * Copyright (c) 2018, Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */
define(['ojs/ojcore',
        'knockout',
        'jquery',
        'viewModels/common_vm_def',
        'models/storagepool',
        'models/administration',
        'models/platform',
        'viewModels/confirmation-dialog',
        'ojs/ojinputtext',
        'ojs/ojbutton',
        'ojs/ojdialog'],
    function (oj, ko, $, CommonVMDefinitons, storagePoolSVC, adminSVC, Platform, confirmation)
    {
        function ClientCommandViewModel()
        {
            var self = this;
            this.clientCommandHelp = ko.observable();

            this.shareName = ko.observable('');
            this.cifsFExpCommands = ko.observable('');
            this.statusMessage = ko.observable('');
            this.nfsCommands = ko.observable('');
            this.cifsCommands = ko.observable('');
            this.allExports = ko.observableArray([]);
            this.hostName = ko.observable('');
            this.hostIP = ko.observable('');
            this.showNfsDiv = ko.observable(true);
            this.showCifsDiv = ko.observable(true);
            /*
             * build up commands for a particular share
             */
            function generateCommands() {

                var samba = '';
                var sambaFExp = '';
                var nfs = '';
                var isNfsAvail = false;
                var isSmbAvail = false;
                var nfs_path = '';

                ko.utils.arrayForEach(self.allExports(), function(exp) {
                    if (exp.type === 'smb') {
                        isSmbAvail = true
                        samba += 'net use <new local drive letter>: \\\\' + self.hostIP() + '\\' + exp.resource + ' /Persistent:Yes\n';
                        sambaFExp += '\\\\' + self.hostIP() + '\\' + exp.resource + '\n';
                    }
                    else if ( exp.type === 'nfs' && !isNfsAvail) {
                        isNfsAvail = true;
                        nfs_path = exp.path;
                    }
                });
                // For NFS we don't need separate command for each exports, for a single share command is same unlike Samba
                if (isNfsAvail) {
                    nfs  = 'mount -t nfs ' + self.hostIP() + ':' + nfs_path + ' <your mount point>';
                }
                else {
                    nfs = 'No NFS exports available';
                }
                if (!isSmbAvail) {
                    samba = 'No SMB export available';
                    sambaFExp = samba;
                }
                self.nfsCommands(nfs);
                self.cifsCommands(samba);
                self.cifsFExpCommands(sambaFExp);
            };

            /**
             * method called to open client commands dialog
             * @public
             */
            this.open = function (name, exp)
            {
                self.shareName(name);
                /*
                 * If this comes from export time we have the parameter exp so we don't need to retrieve the exports, we can just
                 * use the export.
                 * Based on the export type we can hide the specific div. If the type is smb we might want to hide nfs div,
                 * if the export type is nfs we might want to hide the samba div. if this call come from storage page we have to
                 * retrieve the export for the particular share and show both nfs and samba div.
                 */
                if (exp != null)
                {
                    self.allExports.removeAll();
                    self.allExports.push(exp);
                    if (exp.type == 'smb'){
                        self.showNfsDiv(false);
                        self.showCifsDiv(true);
                    }
                    else {
                        self.showCifsDiv(false);
                        self.showNfsDiv(true);
                    }
                    var p = new Platform();

                    p.getInfos().then( function (info) {
                        self.hostName(info.hostname);
                        self.hostIP(info.ip);
                        generateCommands(exp);
                        $(self.dialogSelector).ojDialog('open');
                    }).catch(function (reason) {
                        oj.Logger.warn('geting appliance info has failed : %s', reason);
                    });

                }
                else {
                    self.showCifsDiv(true);
                    self.showNfsDiv(true);

                    storagePoolSVC.getShareExports(name).then( function (list) {
                            self.allExports(list);
                            var p = new Platform();

                            p.getInfos().then( function (info) {
                                self.hostName(info.hostname);
                                self.hostIP(info.ip);
                                generateCommands(exp);
                                $(self.dialogSelector).ojDialog('open');
                            }).catch(function (reason) {
                                oj.Logger.warn('geting appliance info has failed : %s', reason);
                            });

                    }).catch ( function(why) {
                        oj.Logger.warn('cannot get share\'s exports : '+ why);
                    });
                }
            };

            /*
             * Copy the text to clipboard. This function is called by three button click event.
             */
            function copyToClipBoard(value) {
                // create a textarea for now to conduct copy operation
                var textArea = document.createElement("textarea");
                textArea.setAttribute("id", "textAreaForCopy");
                textArea.value = value;
                document.body.appendChild(textArea);
                textArea.focus();
                textArea.select();
                try {
                    var successful = document.execCommand('copy');
                    var msg = successful ? 'successful' : 'unsuccessful';
                    oj.Logger.warn('Copying to clipboard was %s', msg)
                } catch (err) {
                    oj.Logger.warn('Failed to copy client comamnd: %s', err);
                }
                document.body.removeChild(textArea);
            };

            /**
             * Copy nfs command to clipboard
             */
            this.handleCopyClickNfs = function ()
            {
                copyToClipBoard(self.nfsCommands());
            };

            /**
             * Copy samba command to clipboard
             */
            this.handleCopyClickCifs = function ()
            {
                copyToClipBoard(self.cifsCommands());
            };

            /**
             * Copy samba file explorer URL to clipboard
             */
            this.handleCopyClickCifsFExp = function ()
            {
                copyToClipBoard(self.cifsFExpCommands());
            };

            this.handleCancelClick = function ()
            {
                $(self.dialogSelector).ojDialog("close");
            };

            this.handleCloseDrawerClick = function ()
            {
                $(self.dialogSelector).ojDialog("close");
            };
        }

        $.extend(ClientCommandViewModel.prototype,CommonVMDefinitons.prototype.definitions);

        ClientCommandViewModel.prototype.dialogSelector='#clientCommandsDialog';
        ClientCommandViewModel.prototype.dialogTitle=oj.Translations.getTranslatedString('Share Mount and Map Instructions');
        ClientCommandViewModel.prototype.copyLabel = oj.Translations.getTranslatedString('Copy');
        ClientCommandViewModel.prototype.updateDialogTitle = oj.Translations.getTranslatedString('Show client side commands');
        ClientCommandViewModel.prototype.nfsCommandsLabel = oj.Translations.getTranslatedString('NFS Export Mount Commands');
        ClientCommandViewModel.prototype.cifsCommandsLabel = oj.Translations.getTranslatedString('SMB Export Map Commands');
        ClientCommandViewModel.prototype.cifsFExpCommandsLabel = oj.Translations.getTranslatedString('SMB File Explorer URL');
        ClientCommandViewModel.prototype.nfsCommandsDesc = oj.Translations.getTranslatedString('Use the following command on a client instance to access the share exported through NFS. Replace <your mount point> with the path to an empty directory.');
        ClientCommandViewModel.prototype.cifsCommandsDesc = oj.Translations.getTranslatedString('Use the following command line on a Windows client instance to access the share exported through SMB. Replace <new local drive letter>: with an unused drive letter on the client instance.');
        ClientCommandViewModel.prototype.cifsFExpCommandsDesc = oj.Translations.getTranslatedString('Use the following URL on the Windows File Explorer to access this share.');

        return new ClientCommandViewModel();
    }
);
