/**
 * Copyright (c) 2017, Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */

/*
 * share restore viewModel
 * bind with restore-share-dialog.html view
 */

define(['ojs/ojcore',
        'knockout',
        'jquery',
        'viewModels/common_vm_def',
        'models/backups',
        'viewModels/confirmation-dialog',
        'ojs/ojinputtext',
        'ojs/ojdialog'],
       function (oj, ko, $, CommonVMDefinitons, backupsSVC, confirmation)
    {
        function RestoreShareViewModel()
        {
            var self = this;


            this.cancelButtonDisabled =  ko.observable(false);

            this.validationTracker = ko.observable();
            this.shouldDisableRestoreButton = ko.pureComputed(function () {
                var trackerObj = ko.utils.unwrapObservable(self.validationTracker);
                return trackerObj ? (trackerObj.invalidShown || trackerObj.invalidHidden) : false;
            });


            this.namespace = ko.observable();
            this.bucketName = ko.observable();
            this.objectName = ko.observable();
            this.share = ko.observable({});

            /* dedicated to confirmation dialog */
            this.innerDialogTitle = ko.observable('');
            this.innerDialogMessage = ko.observable('');

            this.handleInnerClick = function ()
            {
                $(self.innerDialogSelector).ojDialog("close");
                $(self.dialogSelector).ojDialog("close");
            };

            /**
             * method called to open share's backup restoration dialog
             * @param entry: share entry object to be used to prepopulate dialog box
             * @public
             */
            this.open = function (entry)
            {
                self.share(entry);
                self.namespace('');
                self.bucketName('');
                self.objectName('');
                $('#rosnsi').ojInputText("reset");
                $('#rosbni').ojInputText("reset");
                $('#boni').ojInputText("reset");
                $(self.dialogSelector).ojDialog('open');
            };

            this.handleCancelClick = function ()
            {
                $(self.dialogSelector).ojDialog("close");
            };
            this.handleCloseDrawerClick = function ()
            {
                $(self.dialogSelector).ojDialog("close");
            };

            /**
             * Restore a share depending on the
             * current dialog object actionType
             */
            this.handleRestoreClick = function (data, event)
            {
                backupsSVC.restoreShareFromObjectStore(
                    self.share().name,
                    self.namespace().trim(),
                    self.bucketName().trim(),
                    self.objectName().trim()).then (function (payload)
                              {
                                  oj.Logger.info('Restore succeded');
                              }).catch(function (why) {
                                  oj.Logger.error('Restore has failed: ' + why);
                                  self.innerDialogTitle(self.innerDialogKoTitle);
                                  self.innerDialogMessage('backup has failed: ' + why);
                                  $(self.innerDialogSelector).ojDialog("open");
                              });
                $(self.dialogSelector).ojDialog("close");
            };

        }

        $.extend(RestoreShareViewModel.prototype,CommonVMDefinitons.prototype.definitions);

        RestoreShareViewModel.prototype.optionRequiredHint = oj.Translations.getTranslatedString('You must enter a value');
        RestoreShareViewModel.prototype.initRestoreFailed = oj.Translations.getTranslatedString('Restore has failed');

        RestoreShareViewModel.prototype.dialogSelector='#restoreShareDialog';

        RestoreShareViewModel.prototype.innerDialogSelector='#innerrestoreShareDialog';

        RestoreShareViewModel.prototype.dialogTitle=oj.Translations.getTranslatedString('Restore Share');
        RestoreShareViewModel.prototype.restoreNSHelpDef=oj.Translations.getTranslatedString('Namespace associated with the bucket. Can be found in the Bucket Details page on the OCI console. Click for more info.');
        RestoreShareViewModel.prototype.restoreNSPlaceholder=oj.Translations.getTranslatedString('Namespace associated with the bucket');
        RestoreShareViewModel.prototype.restoreNSHelpSource='https://docs.us-phoenix-1.oraclecloud.com/Content/Object/Tasks/managingobjects.htm?Highlight=namespace';
        RestoreShareViewModel.prototype.restoreBNHelpDef=oj.Translations.getTranslatedString('Bucket name containing the archive object');
        RestoreShareViewModel.prototype.restoreBNPlaceholder=oj.Translations.getTranslatedString('Bucket name containing the archive object');
        RestoreShareViewModel.prototype.restoreBNHelpSource='https://docs.us-phoenix-1.oraclecloud.com/Content/Object/Tasks/managingbuckets.htm';

        RestoreShareViewModel.prototype.OSNSLabel=oj.Translations.getTranslatedString('Object Store namespace');
        RestoreShareViewModel.prototype.OSBNLabel=oj.Translations.getTranslatedString('Object Store bucket name');
        RestoreShareViewModel.prototype.objectNameLabel=oj.Translations.getTranslatedString('Archive Object name');
        RestoreShareViewModel.prototype.restoreHelp=oj.Translations.getTranslatedString('Restore a previously archived share from an OCI object store object. The object will be downloaded and overwrite the current content of the share.');
        RestoreShareViewModel.prototype.restoreObjectNamePlaceholder=oj.Translations.getTranslatedString('Name of the archive object to restore');
       RestoreShareViewModel.prototype.restoreOnHelpDef=oj.Translations.getTranslatedString('Name of the existing archive object to restore');
        return new RestoreShareViewModel();
    }
);
