/**
 * Copyright (c) 2017, Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */
define(['ojs/ojcore',
        'knockout',
        'jquery',
        'viewModels/common_vm_def',
        'models/administration',
        'viewModels/confirmation-dialog',
        'models/platform',
        'ojs/ojinputtext',
        'ojs/ojlabel',
        'ojs/ojdialog',
        'ojs/ojswitch',
        'ojs/ojvalidation-base'],
       function (oj, ko, $, CommonVMDefinitons, adminSVC, confirmation, Platform)
    {
        function SambaGlobalSettingViewModel()
        {
            var self = this;
            this.workgroupName = ko.observable('');
            this.netbiosName = ko.observable('');
            this.interfaces = ko.observable('');
            this.activeDirectoryOption = ko.observable(true);
            this.adcFQDN = ko.observable('');
            this.adcIPAddress = ko.observable('');
            this.adcUserName = ko.observable('');
            this.adcUserPass = ko.observable('');

            this.applianceHostname = ko.observable('');

            this.open = function ()
            {
                if (self.applianceHostname() === '') {
                    new Platform().getInfos().then(function(info) {
                        self.applianceHostname(info.hostname);
                    });
                }
                new adminSVC().getSambaOptions().then(function(options)
                                {
                                    reset_all_info();
                                    self.workgroupName(options['workgroup']);
                                    self.netbiosName(options['netbios name']);
                                    self.interfaces(options['interfaces']);
                                    self.activeDirectoryOption(options['ad_enabled']);
                                    self.adcFQDN(options['fqdn']);
                                    self.adcIPAddress(options['ip_address']);
                                    self.adcUserName(options['user_name']);

                                    $('#workgroup-name-text-input').ojInputText("reset");
                                    $('#netbios-name-text-input').ojInputText("reset");
                                    $('#interfaces-text-input').ojInputText("reset");
                                    $(self.dialogSelector).ojDialog('open');
                                }).catch(function (why)
                                         {
                                             oj.Logger.error('cannot get samba config');
                                             confirmation.showDialog(
                                                 {
                                                     'notifyOnly':true,
                                                     'title': oj.Translations.getTranslatedString('Successful Modification'),
                                                     'message': oj.Translations.getTranslatedString('Fail to get Samba configuration: ' + why)
                                                 });
                                         }
                                        );
            };

            /*
                reset all the parameters to empty value
             */
            function reset_all_info() {
                self.adcFQDN('');
                self.adcIPAddress('');
                self.adcUserName('');
                self.adcUserPass('');
                self.workgroupName('');
                self.netbiosName('');
                self.interfaces('');


            };

            // validator for work group
            this.workgroupValidator = {
                getHint: function () {
                    return self.workgroupValidationHintMessage;
                }
            };

            // Validator for netbios name
            this.netbiosValidator = {
                getHint: function () {
                    return self.netbiosValidationHintMessage;
                },
                errorRaised :  ko.observable(false),
                validate: function(value) {
                    this.errorRaised(false);
                    if (value != undefined && value.length > 15) {
                        this.errorRaised(true);
                        throw new oj.ValidatorError(self.netbiosValidationHintMessage);
                    }
                    if (value === null || value.length === 0) {
                        this.errorRaised(true);
                        throw new oj.ValidatorError('Netbios name can not be empty');
                    }
                    return true;
                }
            };

            // Validator for fully qualified domain name (FQDN)
            this.fqdnValidator = {
                getHint: function () {
                    return self.fqdnValidationHintMessage;
                },
                errorRaised :  ko.observable(false),
                validate: function(value) {
                    var re = new RegExp(/^((?:(?:(?:\w[\.\-\+]?)*)\w)+)((?:(?:(?:\w[\.\-\+]?){0,62})\w)+)\.(\w{2,6})$/);
                    var correctFormat = re.test(value);
                    this.errorRaised(false);
                    if (!correctFormat) {
                        this.errorRaised(true);
                        throw new oj.ValidatorError(self.fqdnValidationHintMessage);
                    }
                    return true;
                }
            };

            // Validator for IP address
            this.ipValidator = {
                getHint: function () {
                    return self.ipValidationHintMessage;
                },
                errorRaised :  ko.observable(false),
                validate: function(value) {
                    var re = new RegExp(/^((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))$/);
                    var correctFormat = re.test(value);
                    this.errorRaised(false);
                    if (!correctFormat) {
                        this.errorRaised(true);
                        throw new oj.ValidatorError(self.ipValidationHintMessage);
                    }
                    return true;
                }
            };

            // validator for User name
            this.userNameValidator = {
                getHint: function () {
                    return self.userNameValidationHintMessage;
                },
                errorRaised :  ko.observable(false),
                validate: function(value) {
                    this.errorRaised(false);
                    if (value === null) {
                        this.errorRaised(true);
                        throw new oj.ValidatorError(self.userNameValidationHintMessage);
                    }
                    if (value.length === 0) {
                        this.errorRaised(true);
                        throw new oj.ValidatorError(self.userNameValidationHintMessage);
                    }
                    return true;
                }
            };

            // Validator for password
            this.passwordValidator = {
                getHint: function () {
                    return self.passwordValidationHintMessage;
                },
                errorRaised :  ko.observable(false),
                validate: function(value) {
                    this.errorRaised(false);
                    if (value === null) {
                        this.errorRaised(true);
                        throw new oj.ValidatorError(self.passwordValidationHintMessage);
                    }
                    if (value.length === 0) {
                        this.errorRaised(true);
                        throw new oj.ValidatorError(self.passwordValidationHintMessage);
                    }
                    return true;
                }
            };

            // Validator for  interfaces
            this.interfacesValidator = {
                getHint: function () {
                    return self.interfacesValidationHintMessage;
                }
            };

            // Check if we should disable the update button based on the user input
            this.shouldDisableCreateButton = ko.pureComputed(function () {
               if (self.activeDirectoryOption()) {
                    if (self.fqdnValidator.errorRaised() === true || self.ipValidator.errorRaised() === true || self.userNameValidator.errorRaised() == true || self.passwordValidator.errorRaised() == true) {
                        return true;
                    }
                    if (self.adcFQDN() === '' || self.adcIPAddress() === '' || self.adcUserName() === '' || self.adcUserPass() === '') {
                        return true;
                    }
                }
                if (self.netbiosName() == '') {
                    return true;
                }
                if (self.netbiosValidator.errorRaised() === true) {
                    return true;
                }
                return false;
            });

            this.handleCancelClick = function ()
            {
                reset_all_info();
                $(self.dialogSelector).ojDialog("close");
            };

            this.handleCloseDrawerClick = function ()
            {
                reset_all_info();
                $(self.dialogSelector).ojDialog("close");
            };

            this.handleUpdateClick = function (data, event)
            {
                confirmation.showDialog({
                       'title':self.updateDialogTitle,
                       'message':self.updateDialogText,
                       'onOK' : function () {
                           var options = {};
                           options['workgroup'] = self.workgroupName();
                           if (self.activeDirectoryOption()) {
                               options['netbios name'] = self.applianceHostname();
                           } else{
                               options['netbios name'] = self.netbiosName();
                           }
                           options['interfaces'] = self.interfaces();
                           options['fqdn'] = self.adcFQDN();
                           options['ip_address'] = self.adcIPAddress();
                           options['user_name'] = self.adcUserName();
                           options['user_password'] = self.adcUserPass();
                           options['ad_enabled']= self.activeDirectoryOption();
                           new adminSVC().setSambaOptions(JSON.stringify(options)).then (function (payload)
                                          {
                                              oj.Logger.info('Samba global setting was updated successfully');
                                              confirmation.showDialog({
                                                'notifyOnly':true,
                                                'title': oj.Translations.getTranslatedString('Successful Modification'),
                                                'message': oj.Translations.getTranslatedString('Samba global setting was updated successfully')
                                            });

                                          }).catch(function (why) {
                                              oj.Logger.error('Updating samba global setting failed: ' + why);
                                              confirmation.showDialog({
                                                  'notifyOnly':true,
                                                  'title': oj.Translations.getTranslatedString('Samba global setting modification failure'),
                                                  'message': why
                                              });
                                            }
                                          );
                            $(self.dialogSelector).ojDialog("close");
                        }
                });
            };
        }

        $.extend(SambaGlobalSettingViewModel.prototype,CommonVMDefinitons.prototype.definitions);

        SambaGlobalSettingViewModel.prototype.dialogSelector='#sambaGlobalSettingDialog';
        SambaGlobalSettingViewModel.prototype.workgroupOptionLabel=oj.Translations.getTranslatedString('Workgroup name');
        SambaGlobalSettingViewModel.prototype.workgroupHelp=oj.Translations.getTranslatedString('Specify the workgroup that the appliance will be added to.  Click for more information.');
        SambaGlobalSettingViewModel.prototype.workgroupValidationHintMessage=oj.Translations.getTranslatedString('Optional and can be left empty');
        SambaGlobalSettingViewModel.prototype.netbiosOptionLabel=oj.Translations.getTranslatedString('NetBIOS name');
        SambaGlobalSettingViewModel.prototype.netbiosNameHelp=oj.Translations.getTranslatedString('Name used to designate the appliance in a windows network. Click for more information. When Active Directory is enabled, the name has to be set to the current hostname');
        SambaGlobalSettingViewModel.prototype.netbiosNamePlaceHolder=oj.Translations.getTranslatedString('Name of the Samba server');
        SambaGlobalSettingViewModel.prototype.netbiosValidationHintMessage=oj.Translations.getTranslatedString('NetBIOS name cannot exceed 15 characters.');
        SambaGlobalSettingViewModel.prototype.interfacesOptionLabel=oj.Translations.getTranslatedString('Network interfaces');
        SambaGlobalSettingViewModel.prototype.interfacesHelp=oj.Translations.getTranslatedString('List of network interface names that Samba will use for browsing. Click for more information');
        SambaGlobalSettingViewModel.prototype.interfacesPlaceHolder=oj.Translations.getTranslatedString('e.g.: eth0 192.168.2.10/24');
        SambaGlobalSettingViewModel.prototype.interfacesValidationHintMessage=oj.Translations.getTranslatedString('Comma separated list of network interface names that Samba will use for browsing.  When this field is left empty, all interfaces are used');
        SambaGlobalSettingViewModel.prototype.dialogTitle=oj.Translations.getTranslatedString('Samba Global Settings');
        SambaGlobalSettingViewModel.prototype.sambaGlobalUpdateLabel=oj.Translations.getTranslatedString('Update');
        SambaGlobalSettingViewModel.prototype.updateDialogTitle = oj.Translations.getTranslatedString('Samba global setting update');
        SambaGlobalSettingViewModel.prototype.updateDialogText = oj.Translations.getTranslatedString('Are you sure you want to update Samba global settings ?');
        SambaGlobalSettingViewModel.prototype.SambaSource = 'https://www.samba.org/samba/docs/man/manpages-3/smb.conf.5.html';

        SambaGlobalSettingViewModel.prototype.activeDirectoryOptionTip = oj.Translations.getTranslatedString('Configure Active Directory settings for Samba access control. Click for more information');
        SambaGlobalSettingViewModel.prototype.activeDirectoryLabel = oj.Translations.getTranslatedString('Enable Active Directory support');
        SambaGlobalSettingViewModel.prototype.activeDirectorySource = 'https://wiki.samba.org/index.php/Setting_up_Samba_as_a_Domain_Member';

        SambaGlobalSettingViewModel.prototype.adcFQDNTip= oj.Translations.getTranslatedString('Fully qualified domain name of the Active Directory server');
        SambaGlobalSettingViewModel.prototype.adcFQDNLabel = oj.Translations.getTranslatedString('Server FQDN');
        SambaGlobalSettingViewModel.prototype.adcFQDNPlaceHolder = oj.Translations.getTranslatedString('e.g.: host.domainname.com');
        SambaGlobalSettingViewModel.prototype.fqdnValidationHintMessage = oj.Translations.getTranslatedString('Specify the fully qualified domain name of the Active Directory server.  For example: host.somedomain.com');

        SambaGlobalSettingViewModel.prototype.adcIPTip= oj.Translations.getTranslatedString('Active Directory server IP address');
        SambaGlobalSettingViewModel.prototype.adcIPLabel = oj.Translations.getTranslatedString('Server IP address');
        SambaGlobalSettingViewModel.prototype.adcIPPlaceHolder = oj.Translations.getTranslatedString('e.g.: 192.168.1.13');
        SambaGlobalSettingViewModel.prototype.ipValidationHintMessage = oj.Translations.getTranslatedString('Enter the full IP address. For example: 192.168.1.22');

        SambaGlobalSettingViewModel.prototype.adcUserTip= oj.Translations.getTranslatedString('Active Directory delegated user name used to join the Active Directory domain');
        SambaGlobalSettingViewModel.prototype.adcUserLabel = oj.Translations.getTranslatedString('Delegated Admin User Name');
        SambaGlobalSettingViewModel.prototype.userNameValidationHintMessage = oj.Translations.getTranslatedString('The user name can not be left empty');

        SambaGlobalSettingViewModel.prototype.adcUserPassTip= oj.Translations.getTranslatedString('Password for the delegated admin Active Directory user');
        SambaGlobalSettingViewModel.prototype.adcUserPassLabel = oj.Translations.getTranslatedString('Delegated Admin Password');
        SambaGlobalSettingViewModel.prototype.passwordValidationHintMessage = oj.Translations.getTranslatedString('Password can not be empty');

        return new SambaGlobalSettingViewModel();
    }
);
