/**
 * Copyright (c) 2017, 2018, Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */
define(['ojs/ojcore', 'knockout','jquery','models/storagepool', 'models/backups', 'models/logs',
         'viewModels/common_vm_def',
        'viewModels/add-share-dialog',
        'viewModels/confirmation-dialog',
        'viewModels/backup-share-dialog',
        'viewModels/restore-share-dialog',
        'viewModels/client-commands-dialog',
        'models/platform',
        'helpers',
        'models/task-helpers',
        'models/storage-types',
        'ojs/ojinputtext',
        'ojs/ojbutton',
        'ojs/ojgauge',
        'ojs/ojtable',
        'ojs/ojpictochart',
        'ojs/ojcolor',
        'ojs/ojprogress'],
       function(oj, ko,$,storagePoolM, backupsM, logM, CommonVMDefinitons,
                addShareDialog, confirmation,
                backupShareDialog, restoreShareDialog, clientCommandsDialog, Platform, Helpers, taskHelpers,
                storageTypeFactory) {


           function StorageViewModel() {
               var self = this;


               this.operationMonitor = undefined;

               this.currentSelectedShareRow = ko.observable();

               this.addShareButtonTitle = ko.observable('');

               this.isAddShareButtonDisabled = ko.pureComputed(function () {
                   if (self.isStorageDisabled()) {
                       self.addShareButtonTitle(self.storageNotAvailable);
                       return true;
                   }
                   if (self.poolFreeSize() === 0) {
                        self.addShareButtonTitle(self.storageFull);
                       return true;
                   }
                   self.addShareButtonTitle('');
                   return false;
               });

               this.isStorageDisabled = ko.pureComputed(function () {
                   var state = self.poolProtectionStatus();
                    if (state == 'active' ||  state == 'degraded') {
                        return false;
                    }
                   return true;
               });

               this.poolDevices = {};

               this.poolProtectionLevel = ko.observable('');
               this.poolProtectionStatus = ko.observable('');
               this.poolProtectionGaugeColor = ko.pureComputed(function() {
                   if (self.poolProtectionStatus() === 'active') {
                       return StorageViewModel.prototype.activeDevState.color.toString();
                   }
                   if (self.poolProtectionStatus() === 'degraded') {
                       return StorageViewModel.prototype.degradedDevState.color.toString();
                   }
                   return StorageViewModel.prototype.faultyDevState.color.toString();
               });
               var protectionToolTipNode = document.createElement('div');
               var protectionToolTipNodespan = document.createElement('span');
               protectionToolTipNode.appendChild(protectionToolTipNodespan);
               this.poolProtectionGaugeToolTip = function() {
                   protectionToolTipNode.textContent =
                       oj.Translations.getTranslatedString(
                           self.poolProtectionLevel() + ' data protection is ' +
                           self.poolProtectionStatus());
                   return  protectionToolTipNode;
               };

               this.poolSize = ko.observable(0);
               this.poolFreeSize = ko.observable(0);

               /* returns the percentage of pool storage consumed */
               this.poolConsumed = ko.pureComputed(function() {
                   if (self.poolSize() > 0) {
                        return (((self.poolSize() - self.poolFreeSize())*100) / self.poolSize());
                   }
                   return 0.0;
               });

               this.poolUsageTooltip = function () {
                   return {
                       insert: "Storage utilization: " +
                           self.byteConverter.format(self.poolFreeSize()) +
                           ' free out of ' +
                           self.byteConverter.format(self.poolSize())
                   };};

               function initPoolUsage(data) {
                      if ('error' in data) {
                        self.poolSize(0);
                        self.poolFreeSize(0);
                        return;
                      }
                      self.poolSize(data.total);
                      self.poolFreeSize(data.free);
               }

               function updatePoolUsage() {
                    storagePoolM.getPoolUsage().then (initPoolUsage);
               }


               this.poolUsageRefresh = undefined;
               this.raidPictoChartItems = ko.observableArray([]);
               this.raidPictoChartColumnCount = ko.observable(0);
               this.raidPictoChartMessage = ko.observable('');
               this.raidPictoChartMessageClass = ko.observable('storage-pool-dev-text');

               this.getReadRequestPerSec = ko.observable(0.0);
               this.getWriteRequestPerSec = ko.observable(13.40);
               this.getReadKBPerSec = ko.observable(5.30);
               this.getWriteKBPerSec = ko.observable(6.60);


               this.shareArray = ko.observableArray([]);
               this.shareDataSource = new oj.ArrayTableDataSource(
                   this.shareArray,
                   {
                       idAttribute: 'name'
                   }
               );
               // for now sorting is not an option
               this.shareDataSource.sortCriteria = {
                   'key': 'name',
                   'direction': 'ascending'
               };
               this.selectedItems = ko.observableArray([]);
               /*
                * Gets the index of the share in self.shareArray()
                * Looking by name
                */
               function _getShareIndexByName(name) {
                   return self.shareArray().map(function(e) { return e.name; }).indexOf(name);
               }

               /**
                * trigger an update on listView datasource
                * @param {string} rowkey - key of lsit row to update
                * @private
                */
               function _triggerUpdateOnSahreListView(rowKey) {
                   self.shareDataSource.get(rowKey).then(function(rowdata) {
                       if (rowdata !== null) {
                           self.shareDataSource.change(rowdata.data).then(
                               function(cdata) {
                                   oj.Logger.log('data source change [' + rowdata.data.name+ '] => state ['+rowdata.data.state+']');
                               });
                       } else {
                           oj.Logger.warn('data source not found ' + rowKey + ']');
                       }
                   });
               }


               /**
                * Monitors background operations on a share.
                * It fetches the current task list and set shares state
                * accordingly
                *
                */
               function shareOperationMonitor() {
                   if (shareOperationMonitor.monitoredTaskList === undefined) {
                       // we have to keep track of tasks including the one
                       // already completed. If we were monitoring them
                       // we have to display last status of them
                       // keep track of runnig task we've tracked
                       shareOperationMonitor.monitoredTaskList = new Set();
                   }

                   var currentRunningTaskList = new Set();
                   new Platform().getRunningTaskList().then(function (ongoingTasks) {
                       // first ignore the one not related to shares we are displaying
                       ko.utils.arrayForEach(ongoingTasks, function (task) {
                           // filter out the one not related to shares we currently display
                           var index = _getShareIndexByName(task.target);
                           if (index !== -1) {
                               oj.Logger.log('running task for target ['+ task.target+ '] id=' +task.ID);
                               currentRunningTaskList.add(task.ID);
                               // otherwise, task is not related, skip it
                               var entry = self.shareArray()[index];
                               oj.Logger.log('target registered for list row change ['+ entry.name + '] state='+
                                                 entry.state);
                           } else {
                               oj.Logger.log('task (id='+task.ID+
                                             ') for unknown target ('+task.target+'), ignore it ');
                           }
                       });
                       // now check for tasks that are not running anymore
                       var notRunningAnymore = new Set();
                       shareOperationMonitor.monitoredTaskList.forEach(function (task_Id) {
                           if (!currentRunningTaskList.has(task_Id)) {
                               oj.Logger.log('task id=' +task_Id + ' not running anymore');
                               // task is not running anymore
                               // if the task has fail flag the UI
                               new Platform().getTask(task_Id).then(function(task){
                                   var entry = self.shareArray()[_getShareIndexByName(task.target)];
                                   if (task.status.code !== 0) {
                                       entry.state = task.status.message ;
                                   } else {
                                       // success, clear the UI
                                       entry.state = '';
                                   }
                                   if (task.name === 'backup') {
                                       entry.lastBackup = taskHelpers.getTaskExecutionEnd(task);
                                   }
                                   oj.Logger.log('target registered for list row change '+
                                                 entry.name+ ' state='+
                                                 entry.state);
                                   _triggerUpdateOnSahreListView(entry.name);
                               }).catch(function (reason) {
                                   oj.Logger.warn('cannot get task detail: %s', reason);
                               });
                               notRunningAnymore.add(task_Id);
                           }
                       });
                       // now update the lists, do not wait for the promise for this
                       currentRunningTaskList.forEach(function (id) {
                           shareOperationMonitor.monitoredTaskList.add(id);
                       });
                       notRunningAnymore.forEach(function (id) {
                           shareOperationMonitor.monitoredTaskList.delete(id);
                       });
                   }).catch (function (raeson) {
                       oj.Logger.warn('cannot get task list: %s', raeson);
                   });
               }

               /*
                * Eventchannel handler for share deletion
                * It is triggered when a share is deleted
                * Parameters:
                *   attributes: a JSON object which contains following KVs
                *      'name': name of the share
                */
               function shareDeleted(attributes) {
                   self.shareArray.remove( function (item) {
                       return (attributes.name === item.name);
                   });
               }

               /*
                * Eventchannel handler for share modification
                * It is triggered when a share is modified
                * Parameters:
                *   attributes: a JSON object which contains following KVs
                *      'name': name of the share
                */
               function shareModified(attributes) {
                   storagePoolM.getShare(attributes.name).then(
                           function (share) {
                               var n_share = createArrayEntry(share);
                               storagePoolM.getShareExports(n_share.name).then(
                                   function(exports) {
                                       n_share.exports = exports;
                                       var index = _getShareIndexByName(attributes.name);
                                       /*we update the share object foudn in data source */
                                       /*with new values  coming from n_share */
                                       /* we keep 'state' from existing share object */
                                       /* as new one do not have any for now*/
                                       $.extend(true,
                                                self.shareArray()[index],
                                                n_share,
                                                {
                                                    'state':self.shareArray()[index].state
                                                });
                                       //n_share.state = self.shareArray()[index].state;
                                       //self.shareArray.splice(index, 1, n_share);
                                       _triggerUpdateOnSahreListView(n_share.name);
                                   });
                           });
               }

               /*
                * Eventchannel handler for share creation
                * It is triggered when a share is created
                * Parameters:
                *   attributes: a JSON object which contains following KVs
                *      'name': name of the share
                *      'tasks_id' : task id of the task to do share share creation
                */
               function newShareCreated (attributes) {
                   oj.Logger.log('new share created event. taskid = ' + attributes.task_id);
                   var _newShare = createArrayEntry(storageTypeFactory.newShare(attributes.name));
                   _newShare.state = oj.Translations.getTranslatedString('Share creation in progress');
                   self.shareArray.push(_newShare);
                   /* explicitly ask for this task to be monitored now */
                   shareOperationMonitor.monitoredTaskList.add(attributes.task_id);

                   /*be sure our list is sorted*/
                   self.shareDataSource.sort();
                   self.selectedItems([_newShare.name]);
                   var listV = $('#shareListView').ojListView()[0];
                   oj.Assert.assert(listV !== undefined);
                   /* now we try to get info about it */
                   /* we may fail as it is in middle of creation */
                   /* do our best effort */
                   storagePoolM.getShare(attributes.name).then(
                       function (s) {
                           $.extend(true,_newShare,s,{
                               'state':_newShare.state});
                       });

                   /* $('#shareListView').ojListView()[0].scrollToItem(
                    {'key':n_share.name}); */
               }

               /*
                * Eventchannel handler for share backup and restore
                * It is triggered when a share backup or restore is called
                * Parameters:
                *   data: a JSON object which contains following KVs
                *      'taskId' : task id of the task to do share backup or restore
                */
               function shareBackupRestore(data) {
                   new Platform().getTask(data.taskId).then(function(task){
                       var entry = self.shareArray()[_getShareIndexByName(task.target)];
                       oj.Logger.log('share ' +  task.name + 'event. taskid = ' + data.taskId);
                       if (task.name === 'backup') {
                           entry.state = self.backingUpShareMessage;
                       } else if (task.name === 'restore') {
                           entry.state = self.RestoringShareMessage;
                       }
                       shareOperationMonitor.monitoredTaskList.add(data.taskId);
                       self.shareDataSource.sort();
                       self.selectedItems([entry.name]);
                       var listV = $('#shareListView').ojListView()[0];
                       oj.Assert.assert(listV !== undefined);
                   }).catch(function (reason) {
                       oj.Logger.warn('cannot get task detail: %s', reason);
                   });
               }

               storagePoolM.shareCreationEventchannel.add(newShareCreated);
               storagePoolM.shareModificationEventchannel.add(shareModified);
               storagePoolM.shareDeletionEventchannel.add(shareDeleted);
               storagePoolM.shareBackupRestoreEventchannel.add(shareBackupRestore);

               storagePoolM.shareCreationEventchannel.add(updatePoolUsage);
               storagePoolM.shareModificationEventchannel.add(updatePoolUsage);
               storagePoolM.shareDeletionEventchannel.add(updatePoolUsage);

               /* returns an object ready to be added on shareArray */
               /* i.e we extend the object to attach what's needed in the UI*/
               function createArrayEntry(share) {
                   return $.extend(true,{},
                                   share,
                                   {
                                       'used': share.size - share.free,
                                       'state': '',
                                       'lastBackup': ''
                                   });
               }

               /**
                * return the template to be used to realiaze a row in the
                * storage list view.
                */
               this.getShareTemplate = function (itemContext) {
                   var share = itemContext.data;

                   if (share.state === '') {
                       return oj.KnockoutTemplateUtils.getRenderer('share_template', true)(itemContext);
                   }
                   return oj.KnockoutTemplateUtils.getRenderer('share_under_operation_template', true)(itemContext);
               };

               this.handleDeactivated = function () {
                    window.clearInterval(self.operationMonitor);
                    window.clearInterval(self.poolUsageRefresh);
                };
               this.handleActivated = function(info)
               {

                   self.shareArray.removeAll();
                   storagePoolM.getShares().then(
                       function (data) {
                           // TODO : for now we flush the share array
                           // later may be better to just update it
                           var shareOnlyList=data.map(function(share) {
                               return createArrayEntry(share);
                           });
                           shareOnlyList.forEach(function (share) {
                               storagePoolM.getShareExports(share.name).then(function(exports) {
                                   share.exports = exports;
                                   storagePoolM.geShareTaskTypeList(share.name,'backup').then(
                                       function(tl) {
                                           share.lastBackup = taskHelpers.getMostRecentTaskExecutionEnd(tl);
                                           self.shareArray.push(share);
                                       }).catch(function (why) {
                                           self.shareArray.push(share);
                                           oj.Logger.warn('cannot get backup time ' + why);
                                       });
                               });
                           });
                       });

                   self.shareDataSource.sort();

                   self.poolUsageRefresh = window.setInterval(updatePoolUsage, 600000);
                   /* make sure the value is up te date  */
                   storagePoolM.getPoolUsage().then(initPoolUsage);

                   //TODO : treat errors
                   storagePoolM.getPoolStatus().then(function (data) {
                       var activeDevCount=0;
                       var deviceCount=data.devices.length;
                       var num_raid_devs = data.num_devs;
                       var items=[];

                       self.poolProtectionStatus(data.status);
                       self.poolDevices = [];
                       self.poolProtectionLevel(data.protection);

                       for (var i = 0; i < deviceCount; i++) {
                           /*
                            * In items computed below we specify color
                            * when using 'source' attribute, color attribute is ignored
                            * we still use it as it is used in tooltip renderer
                            */
                           self.poolDevices[data.devices[i].dev] = data.devices[i];
                           if (data.devices[i].state === 'active') {
                               activeDevCount += 1;
                               items.push({'name': data.devices[i].dev,
                                           'source': self.images + 'qual_rectanglesthree_24_active.png',
                                           'color': self.activeDevState.color.toString()});
                           }
                           if (data.devices[i].state === 'faulty') {
                               items.push({'name': data.devices[i].dev,
                                           'source': self.images + 'qual_rectanglesthree_24_faulty.png',
                                           'color': self.faultyDevState.color.toString()});
                           }
                           if (data.devices[i].state === 'spare') {
                               items.push({'name': data.devices[i].dev,
                                           'source': self.images + 'qual_rectanglesthree_24_spare.png',
                                           'color': self.spareDevState.color.toString()});
                           }
                           if (data.devices[i].state === 'removed') {
                               items.push({'name': data.devices[i].dev,
                                           'source': self.images + 'qual_rectanglesthree_24_removed.png',
                                           'color': self.removedDevState.color.toString()});
                           }
                       }

                       self.raidPictoChartColumnCount(deviceCount);
                       self.raidPictoChartItems(items);
                       self.raidPictoChartMessage(activeDevCount + ' out of ' + num_raid_devs +
                                                  ((num_raid_devs > 1) ? ' devices' : ' device') +
                                                  ((activeDevCount > 1) ? ' are' : ' is') +
                                                  ' active');
                       if (activeDevCount !== num_raid_devs) {
                           self.raidPictoChartMessageClass('storage-pool-dev-warning-text');
                       } else {
                           self.raidPictoChartMessageClass('storage-pool-dev-text');
                       }
                   }).catch(function (reason)
                            {
                                oj.Logger.warn('get pool usage has failed : %s',reason);
                            });

                   // be sure first update is done right away
                   shareOperationMonitor();
                   self.operationMonitor = window.setInterval(shareOperationMonitor,10000);
               };

               function deleteShare(name) {
                   storagePoolM.deleteShare(name).catch (
                       function (why) {
                           confirmation.showDialog({
                               'notifyOnly':true,
                               'title':StorageViewModel.prototype.deleteLabel,
                               'message':why
                           });
                       });
               }
               /**
                * Display the appropriate version of the addShareDialog
                * depending on the item selected on the popup menu
                */
               this.shareActionSelect = function (event) {
                   var action = event.target.value;
                   var shareName = self.currentSelectedShareRow();
                   var shareArray = self.shareArray();
                   var share;
                   shareArray.forEach(function(e) {
                       if (e.name == shareName) {
                           share = e;
                       }
                   });
                   var shareList = shareArray.map(function(e) { return e.name; });
                   switch (action) {
                    case 'delete':
                        confirmation.showDialog({
                            'title':StorageViewModel.prototype.deleteLabel,
                            'message':oj.Translations.getTranslatedString('Are you sure you want to delete the share named: ') +shareName + oj.Translations.getTranslatedString(' and its associated data ?'),
                            'onOK' : function () {
                                deleteShare(shareName);
                                }
                        });
                        break;
                    case 'modify':
                       addShareDialog.open(
                           self.poolFreeSize(),
                           shareList,
                           'modify',
                           share);
                       break;
                   case 'duplicate':
                       addShareDialog.open(
                           self.poolFreeSize(),
                           shareList,
                           'duplicate',
                           share);
                       break;
                   case 'backup':
                       backupShareDialog.open(share);
                       break;
                   case 'restore':
                       restoreShareDialog.open(share);
                       break;
                   case 'clientcmd':
                       clientCommandsDialog.open(share.name, null);
                       break;
                   default:
                       oj.Assert.assertString(undefined, 'action not defined !');
                   }
               };
               this.addShareClick = function (event) {
                   //pass list of existing share names used by validator
                   addShareDialog.open(
                       self.poolFreeSize(),
                       self.shareArray().map( function(entry) {
                           return entry.name;
                       }),
                       'create',
                       null);
               };
               this.shareUsageTooltip = function (dataContext) {
                   // see view. the element id is computed with share name
                   var shareName = dataContext.componentElement.id.substr('gauge_'.length);
                   var share;
                   self.shareArray().forEach(function(e) {
                       if (e.name == shareName) {
                           share = e;
                       }
                   });
                   var size = self.byteConverter.format(share.size);
                   var free = self.byteConverter.format(share.free);
                   var used = self.byteConverter.format(share.size - share.free);
                   return {insert: "Used : " + used + '<br>Size: ' + size +
                           ' free: ' + self.percentConverter.format((share.free/share.size)) + ' ('+free+')'};
               };
               this.poolStatusDeviceTooltipRenderer = function (dataContext) {
                   if (self.tooltipElem === undefined) {
                       self.tooltipElem = document.createElement('div');
                       var devNameDiv = document.createElement('div');
                       self.tooltipElemName = document.createElement('span');
                       self.tooltipElemName.class = 'storage-pool-tooltip-dev-name';
                       devNameDiv.appendChild(self.tooltipElemName);
                       devNameDiv.appendChild(document.createElement('br'));

                       self.devWWNDiv = document.createElement('div');
                       self.tooltipElemWWN = document.createElement('span');
                       self.tooltipElemWWN.class = 'storage-pool-tooltip-dev-name';
                       self.devWWNDiv.appendChild(self.tooltipElemWWN);
                       self.devWWNDiv.appendChild(document.createElement('br'));

                       var devStateDiv = document.createElement('div');
                       self.tooltipElemState = document.createElement('span');
                       self.tooltipElemState.class = 'storage-pool-tooltip-dev-state';
                       devStateDiv.appendChild(self.tooltipElemState);

                       self.tooltipElem.appendChild(devNameDiv);
                       self.tooltipElem.appendChild(self.devWWNDiv);
                       self.tooltipElem.appendChild(devStateDiv);
                   }

                   self.tooltipElemName.textContent = dataContext.name;
                   if (self.poolDevices[dataContext.name].wwn !== undefined &&
                       self.poolDevices[dataContext.name].wwn !== null &&
                       self.poolDevices[dataContext.name].wwn.length > 0 )
                   {
                       self.tooltipElemWWN.textContent = self.poolDevices[dataContext.name].wwn;
                       self.devWWNDiv.style.display = 'block';
                   } else {
                       self.devWWNDiv.style.display = 'none';
                   }
                   self.tooltipElemState.textContent = self.poolDevices[dataContext.name].state;

                   return self.tooltipElem;
           };

           }

           $.extend(StorageViewModel.prototype,CommonVMDefinitons.prototype.definitions);
           $.extend(StorageViewModel.prototype,CommonVMDefinitons.prototype.converters);
           $.extend(StorageViewModel.prototype,CommonVMDefinitons.prototype.helpers);
           $.extend(StorageViewModel.prototype,CommonVMDefinitons.prototype.paths);

           StorageViewModel.prototype.tooltipElem = undefined;
           StorageViewModel.prototype.tooltipElemName = undefined;
           StorageViewModel.prototype.tooltipElemState = undefined;


           StorageViewModel.prototype.sharesLabel=oj.Translations.getTranslatedString('Shares');

           StorageViewModel.prototype.ACTION_LABEL='Actions';

           StorageViewModel.prototype.backingUpShareMessage = oj.Translations.getTranslatedString('backing up...');
           StorageViewModel.prototype.RestoringShareMessage = oj.Translations.getTranslatedString('restoring content...');


           StorageViewModel.prototype.activeDevState =
               {
                   'color': new oj.Color("#81BB5F"),
                   'name':'active'
               };
           StorageViewModel.prototype.degradedDevState =
               {
                   'color': new oj.Color("#DEB887"),
                   'name':'degraded'
               };
           StorageViewModel.prototype.faultyDevState =
               {
                   'color': new oj.Color("#E95B54"),
                   'name':'faulty'
               };
           StorageViewModel.prototype.spareDevState =
               {
                   'color': new oj.Color("#84BBE6"),
                   'name': 'spare'
               };
           StorageViewModel.prototype.removedDevState =
               {
                   'color': new oj.Color("FBCE4A"),
                   'name': 'removed'
               };



           StorageViewModel.prototype.LAST_BCK_LABEL = oj.Translations.getTranslatedString('Last backup');
           StorageViewModel.prototype.NO_BCK_LABEL = oj.Translations.getTranslatedString('Not backed up yet');
           StorageViewModel.prototype.EXP_PROTO_LABEL = oj.Translations.getTranslatedString('Export protocols');
           StorageViewModel.prototype.NO_EXP_PROTO_LABEL = oj.Translations.getTranslatedString('Not Exported');

           StorageViewModel.prototype.addShareTitle = oj.Translations.getTranslatedString('Add a new share');
           StorageViewModel.prototype.storageNotAvailable = oj.Translations.getTranslatedString('Underlying storage is not available');
           StorageViewModel.prototype.storageFull = oj.Translations.getTranslatedString('No free space available');

           StorageViewModel.prototype.getAllEnabledProtocols = function (shareListRowData) {
               var exportTypes = [];
               shareListRowData.exports.forEach(function(e) {
                   var uce = e.type.toUpperCase();
                   if (exportTypes.indexOf(uce) === -1) {
                       exportTypes.push(uce);
                   }
               });

               return exportTypes;
           };

           return new StorageViewModel();
       });
